//
// AustralianShunting.gs
//
//  Copyright (C) 2002 Auran Developments Pty Ltd
//  All Rights Reserved.
//

include "trainz.gs"
include "world1.gs"
include "common.gs"
include "browser.gs"
include "stringtable.gs"


//
// class MyAustraliaShunt
// Scenario class for the Biriburra Yards scenario.
//
game class MyAustraliaShunt isclass Scenario, ScenarioSupport
{
  bool scenarioDone = false;
  Train trainShunter, trainHaulAway, roundHouseLoco1, roundHouseLoco2, indianPacific;
  Signal signalDepot1, signalNorth6Dwarf, signalNorth3, exitSignal3;

  StringTable stringTable; // scenario string table

  // The bad couple flag is used to indicate if a bad coupling operation has occurred in the past
  // several seconds. This is so a bad coupling by the user isn't intepreted and penalized twice as
  // a decoupling operation generates message from both vehicles involved.
  bool recentBadCouple = false;

  // String for storing objective icon name to. This is so once a error message has been displayed,
  // the previous objective icon can be restored.
  string objectiveIcon = null;


  //
  // Load will be called by Trainz to load the scenario map, and when the user presses Ctrl-L
  // param data is the save game data if loading a saved game.
  //
  bool Load(string data)
  {
    if(data and data.size())
    {
      Interface.Load(data);
    }

    // initialize scenario string table
    stringTable = ScenarioStringTable.Get();

    // load the map
    if(!World.LoadMap(World.FindKUID("australia")))
    {
      Interface.Log(stringTable.GetString1("std_error_loading_map_name", "australia"));
      return false;
    }

    // disable saving on this scenario
    BrowserInterface.SetSaveAllowed(false);


    //
    // Create train consists and place on the map
    //

    // User's switching train consist
    trainShunter = World1.CreateTrain("shunttrain", "Arrival Track", true);
//    trainShunter  = World1.CreateTrain("roundhouseloco1", "Start TurnTableEntry", true); // test/debug
//    trainShunter  = World1.CreateTrain("roundhouseloco1", "Track3", true); // test/debug
    trainShunter.SetFloatingLimit(10.0 * Train.KPH_TO_MPS, true);
    trainShunter.SetAutopilotMode(Train.CONTROL_SCRIPT); // disable user control until we are ready to roll later on

    trainShunter.GetFrontmostLocomotive().SetHasDriver(true);

    // wagons that user is to haul away at end of scenario
    trainHaulAway = World1.CreateTrain("haulawaywagons", "Departure Track", true);

    // Locos that sit idle in roundhouse (static scenery of interest)
    roundHouseLoco1 = World1.CreateTrain("roundhouseloco1", "Start TT4", true);
    roundHouseLoco2 = World1.CreateTrain("roundhouseloco2", "Start TT7", true);


    //
    // Junctions
    //

    // Western end of yard (turntable depot and tunnel)
    Navigate.LockJunction("DEPOT1", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("NORTH1", Junction.DIRECTION_LEFT, true);
    Navigate.LockJunction("NORTH3", Junction.DIRECTION_RIGHT, true);

    // eastern end of yard (Eweview concrete bridge)
    Navigate.LockJunction("NORTH5", Junction.DIRECTION_LEFT, true);
    Navigate.LockJunction("NORTH6", Junction.DIRECTION_LEFT, true);
    Navigate.LockJunction("NORTH7", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("NORTH8", Junction.DIRECTION_LEFT, true);

    // private siding access
    Navigate.LockJunction("Biriburra Track", Junction.DIRECTION_RIGHT, true);
    Navigate.LockJunction("Eweview_Bridge", Junction.DIRECTION_RIGHT, true);
    Navigate.LockJunction("Eweview_Switch", Junction.DIRECTION_LEFT, false);


    //
    // Signals
    // 

    // get access to gameobjects of several signals
    signalDepot1      = cast<Signal> Router.GetGameObject("SD1");
    signalNorth6Dwarf = cast<Signal> Router.GetGameObject("S6D");
    signalNorth3      = cast<Signal> Router.GetGameObject("N3");
    exitSignal3       = cast<Signal> Router.GetGameObject("ES1");

    // set Depot1 to red for now (changed later as needed)
    signalDepot1.SetSignalState(Signal.RED, stringTable.GetString("std_signal_wait_until_clear"));

    return true;
  }


  //
  // Save will be called by Trainz when the user presses Ctrl-S.
  // return the save game string, such that load will be able to restor the save game
  // from the last save check point.
  //  
  string Save()
  {
    return Interface.Save();
  }


  //
  // Note: This method isn't being used at the moment.
  //
  // Called when a train has crossed one of the boundaries that define the yard. If it is the user's
  // train, the scenario is terminate.
  //
  void OffLimits(Message msg)
  {
    // verify it was the shunter train
    if (msg.src == trainShunter)
    {
      Interface.SetObjective(stringTable.GetString("terminated_out_of_bounds"), "bad.tga");
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);
    }
  }


  //
  // Called when train has overshot Tilnowle station.
  //
  void FailedStop(Message msg)
  {
    // verify it was the shunter train
    if (msg.dst == trainShunter)
    {
      Interface.SetObjective(stringTable.GetString("terminated_failed_to_stop"), "bad.tga");
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);
    }
  }


  //
  // TrainDerailed() will be called by Trainz when a train derails
  //
  void TrainDerailed(int trainId) 
  {
    if(!scenarioDone)
    {
      Interface.SetObjective(stringTable.GetString("std_terminated_derailed_train"), "bad.tga");
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);
      scenarioDone = true;
    }
  }


  //
  // TrainCollided() will be called by Trainz when a train collides
  //
  void TrainCollided(int trainId) 
  {
    if(!scenarioDone)
    {
      Interface.SetObjective(stringTable.GetString("std_terminated_train_collision"), "bad.tga");       
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);
      scenarioDone = true;
    }
  }


  //
  // TrainSpeedingFine() is called by Trainz every second your trains speed exceeds the floating limit
  //
  void TrainSpeedingFine() 
  {
    Interface.AdjustScore(-1);  
  }


  //
  // Warns user of their bad couple-up and displays a message along with alarm sound.
  //
  thread void BadCouple()
  {
    // set to true so this method won't be called repeatedly in near proximity
    recentBadCouple = true;

    // warn user and deduct points
    Interface.SetObjective(stringTable.GetString("std_bad_couple"), "bad.tga");
    Interface.AdjustScore(-50);
    World.Play2DSound("ALARM.wav");

    // recentDecouple reset after delay so we don't get 2 BadCouple() calls too close to each other
    Sleep(1.0);
    recentBadCouple = false;

    // after brief sleep, hide warning message and restore previous icon
    Sleep(3.0);
    Interface.SetObjective("", objectiveIcon);
    Interface.ShowObjective(false);
  }


  //
  // TrainBadCouple() is called by Trainz when vehicles couple greater than 8kph (5mph).
  //
  void TrainBadCouple(int vehicleId)
  {
    // was it a recent bad couple? If so, display message and alarm
    if (!recentBadCouple) 
      BadCouple();
  }


  //
  // Handles the shunting. Effectively loops until all wagons are in place.
  //
  void DoShunting(ShuntTrack[] shuntTrack, int wagoncount)
  {
    int i;                      // loop counter
    int temp = wagoncount - 1;
    int droppedcount = 0;       // count of dropped-off wagons
    bool shuntingDone = false;  // shunting done stae flag
    string status;              // string that accumelates state of wagons for user display
    Message msg;
    Vehicle shuntLoco = trainShunter.GetFrontmostLocomotive();
    Vehicle tempVehicle;
    Train tempTrain;

    Interface.Log("Start of DoShunting()"); // test/debug


    // loop until shunting is done
    while (!shuntingDone)
    {
      wait() 
      {
        // wait for a decouple message (we only check for decoupled messages, wasteful otherwise)
        on "Vehicle", "Decoupled", msg :
        {
          // sleep a bit so wagons have chance to roll away and not accidentally couple up to train again
          Sleep(0.5f);

          // for each vehicle, if not attached to loco, apply brakes
          for (i = 0; i < wagoncount; i++)
          {
            if (!TrainUtil.IsInTrain(shuntLoco.GetMyTrain(), shuntTrack[i].vehicle))
            {
              tempTrain = shuntTrack[i].vehicle.GetMyTrain();
              tempTrain.SetDCCThrottle(0.0);
              shuntTrack[i].vehicle.SetHandBrake(true);
            }
          }

          // reset various states
          shuntingDone = true;  // set flag to true, any vehicle out of place will make it false
          droppedcount = 0;     // count of dropped cars starts at 0
          status = "";          // reset status string to nothing

          // test each vehicle for their location
          for (i = 0; i < wagoncount; i++)
          {
            // if the wagon isn't in the zone or still attached to the loco, then the shunting obviously isn't done!
            if (!TrainUtil.IsInTrigger(me, shuntTrack[i].vehicle, shuntTrack[i].trigger) or
                TrainUtil.IsInTrain(shuntLoco.GetMyTrain(), shuntTrack[i].vehicle))
            {
              shuntingDone = false;
            }
            else
            {
              // wagon is in its zone, so add info to status stirng and increment dropped car count
              if (i == temp)
                status = status + shuntTrack[i].name;
              else 
                status = status + shuntTrack[i].name + ", ";

              droppedcount++;
            }
          }

          break;
        }
        on "Vehicle", "Coupled", msg :
        {
          tempVehicle = cast<Vehicle> msg.src;
          tempTrain = tempVehicle.GetMyTrain();

          // if the train that has been coupled up to has the loco in it, release handbrake
          if (TrainUtil.IsInTrain(tempTrain, shuntLoco)) 
            tempTrain.SetHandBrake(false);

          break;
        }
      }


      // Display status update of wagons to user (accumelated during wagon check cycle)
      if (droppedcount > 0)
        status = stringTable.GetString3("num_dropped_wagons", status, droppedcount, wagoncount);
      else
        status = stringTable.GetString("no_dropped_wagons");

      Interface.Print(status);
      Interface.Log(status);
    }

    Interface.Log("End of DoShunting()"); // temp/debug
  }


  //
  // Sets the junctions required for the journer from yard to Tilnowle 
  //
  void SetJunctionsTilnowle()
  {
    // from yard to tunnel
    Navigate.LockJunction("NORTH1", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("NORTH3", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("DEPOT1", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Depot Tunnel Jct", Junction.DIRECTION_LEFT, false);

    // yard/station at other tunnel exit
    Navigate.LockJunction("Junction 529", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Junction 472", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Junction 354", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Junction 245", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Junction 482", Junction.DIRECTION_LEFT, false);

    // approach into Tilnowle
    Navigate.LockJunction("Frickenburg MIlls", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Wattamondra", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Wattamondra East1", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Wattamondra West2", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Tilnowle Loop", Junction.DIRECTION_LEFT, false);
  }


  //
  // main thread
  // brief main is executed automatically after Load() is called.  edit
  // main to contain your scenarios gameplay.
  //
  thread void main(void)
  {
    int i, wagoncount;
    Vehicle[] tempArr;
    ShuntTrack[] tracking;
    Message msg;

    // Start the monitor thread to monitor speeding, derailing etc.
    Monitor();


    //
    // Initialise tracking mechanism for wagons
    //
    tempArr = trainShunter.GetVehicles();
    wagoncount = tempArr.size() - 1;

    tracking = new ShuntTrack[wagoncount];
    for (i = 0; i < wagoncount; i++)
      tracking[i] = new ShuntTrack();

    // gondalas
    tracking[0].vehicle = tempArr[1];
    tracking[0].trigger = cast<Trigger> Router.GetGameObject("Trig GondDest");
    tracking[0].name    = "Gondola 1";
    tracking[1].vehicle = tempArr[2];
    tracking[1].trigger = tracking[0].trigger;
    tracking[1].name    = "Gondola 2";

    // flatwagons
    tracking[2].vehicle = tempArr[3];
    tracking[2].trigger = cast<Trigger> Router.GetGameObject("Trig FlatDest");
    tracking[2].name    = "Flat 1";
    tracking[3].vehicle = tempArr[4];
    tracking[3].trigger = tracking[2].trigger;
    tracking[3].name    = "Flat 2";

    // louvered vans
    tracking[4].vehicle = tempArr[5];
    tracking[4].trigger = cast<Trigger> Router.GetGameObject("Trig VanDest");
    tracking[4].name    = "Van 1";
    tracking[5].vehicle = tempArr[6];
    tracking[5].trigger = tracking[4].trigger;
    tracking[5].name    = "Van 2";


    //
    // Intro messages and explanations etc..
    //

    // Start off with 1000 points.  This will give us some points to play with before
    // the user obtains a negative score (if they do things wrong)
    Interface.AdjustScore(1000);

    // display pause icon at start..
    objectiveIcon = "pause.tga";
    Interface.SetObjectiveIcon("pause.tga");

    // Let user select DCC or Cabin Control for the scenario
    SelectCabinDCC(trainShunter);

    // initial display of instructions and start thread which intercepts 'k' key and displays HTML page of instructions
    DoInstructions();
    ManageBrowser();

    // play intro radio message and play icon
    Sleep(Math.Rand(3,5));
    Sleep(World.Play2DSound("CLEAR.WAV"));
    objectiveIcon = "play.tga"; // update icon tracking string
    trainShunter.SetAutopilotMode(Train.CONTROL_MANUAL);
    Interface.SetObjectiveIcon("play.tga");

    // enable coupling and decoupling on user's consist (locked off in config file so they can't 
    // stuff things up while radio message and intro is happening)
    trainShunter.SetCouplingMask(Vehicle.COUPLE_BOTH);
    trainShunter.SetDecouplingMask(Vehicle.COUPLE_BOTH);


    //
    // Handle the shunting
    //

    // method waits until all vehicles are in place
    DoShunting(tracking, wagoncount);

    // now that everything is parked and in place, lock off rolling stock from being messed with again
    for (i = 0; i < wagoncount; i++)
    {
      tracking[i].vehicle.SetCouplingMask(Vehicle.COUPLE_NONE);
      tracking[i].vehicle.SetDecouplingMask(Vehicle.COUPLE_NONE);
    }
    trainShunter.SetCouplingMask(Vehicle.COUPLE_NONE);
    trainShunter.SetDecouplingMask(Vehicle.COUPLE_NONE);

    // 600 points for completing shuynting
    Interface.AdjustScore(600);
    Interface.Print(stringTable.GetString1("aus_objective_finished_shunt", "600"));
    Interface.Log(stringTable.GetString1("aus_objective_finished_shunt", "600"));

    // inform user to head off to the turntable
    objectiveIcon = "play.tga"; // update icon tracking string
    Interface.SetObjectiveIcon("play.tga");
    Interface.Print(stringTable.GetString("aus_order_turntable"));
    Interface.Log(stringTable.GetString("aus_order_turntable"));


    //
    // Take locomotive to turntable and turn it around
    //

    // allow user to get through turn table signal (is Red unless we do it manually)
    signalDepot1.SetSignalState(Signal.YELLOW, stringTable.GetString("signal_turntable_clear"));

    Navigate.OnTrigger(me, trainShunter, "Trig TurnTableEntry", Navigate.TRIGGER_ENTER);
    objectiveIcon = "pause.tga"; // update icon tracking string
    Interface.SetObjectiveIcon("pause.tga");
    signalDepot1.SetSignalState(Signal.RED, stringTable.GetString("signal_do_not_go"));

    // Turntable handler
    TrainUtil.RunTurnTable(me, trainShunter, "Trig TurnTable");


    //
    // Do coupleup with the freight train
    //
    Navigate.LockJunction("DEPOT1", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("NORTH1", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("NORTH3", Junction.DIRECTION_LEFT, false);

    // switch icon from turn-around back to play as user exits turntable
    Navigate.OnTrigger(me, trainShunter, "Trig TurnTableEntry", Navigate.TRIGGER_ENTER);
    objectiveIcon = "play.tga"; // update icon tracking string
    Interface.SetObjectiveIcon("play.tga");

    // 200 points for completing the turntable
    Interface.AdjustScore(200);
    Interface.Print(stringTable.GetString1("aus_objective_done_turntable", "200"));
    Interface.Log(stringTable.GetString1("aus_objective_done_turntable", "200"));

    Sleep(5);
    Interface.Print(stringTable.GetString("aus_order_pickup_consist"));
    Interface.Log(stringTable.GetString("aus_order_pickup_consist"));

    // guide user into the pickup
    Navigate.OnTrigger(me, trainShunter, "Trig HaulStart", Navigate.TRIGGER_ENTER);
    objectiveIcon = "pause.tga"; // update icon tracking string
    Interface.SetObjectiveIcon("pause.tga");

    Navigate.OnTrigger(me, trainShunter, "Trig HaulStart", Navigate.TRIGGER_LEAVE);
    objectiveIcon = "decouple.tga"; // update icon tracking string
    Interface.SetObjectiveIcon("decouple.tga");

    // do the pickup with haul away consist
    TrainUtil.DoCoupleUp(trainShunter, trainHaulAway);

    // sleep for a few seconds and tell user to take train away
    Sleep(3);
    objectiveIcon = "play.tga"; // update icon tracking string
    Interface.SetObjectiveIcon("play.tga");

    // 200 points for picking up the consist
    Interface.AdjustScore(200);
    Interface.Print(stringTable.GetString1("aus_objective_done_pickup", "200"));
    Interface.Log(stringTable.GetString1("aus_objective_done_pickup", "200"));

    Sleep(5);
    Interface.Print(stringTable.GetString("aus_order_to_tilnowle"));
    Interface.Log(stringTable.GetString("aus_order_to_tilnowle"));


    //
    // Take train through to Tilnowle station
    // 
    
    // set signal and junctions for trip from yard to Tilnowle...
    signalDepot1.SetSignalState(Signal.GREEN, stringTable.GetString("signal_tunnel_clear"));
    SetJunctionsTilnowle();
    
    // when in tunnel, change time of day a few hours forward into late afternoon
    Navigate.OnTrigger(me, trainShunter, "TrigInTunnel", Navigate.TRIGGER_ENTER);
    World.SetGameTime(0.225);

    // create indian pacific waiting at tilnowle
    indianPacific = World1.CreateTrain("IndianPacific", "StartIPTil", true);

    // warn user of approach to Tilnowle
    Navigate.OnTrigger(me, trainShunter, "TrigWarnStop", Navigate.TRIGGER_ENTER);
    objectiveIcon = "pause.tga"; // update icon tracking string
    Interface.SetObjectiveIcon("pause.tga");

    // wait for user to pull into Tinowle and stop
    Navigate.OnTrigger(me, trainShunter, "TrigStopFreight", Navigate.TRIGGER_ENTER);
    objectiveIcon = "stop.tga"; // update icon tracking string
    Interface.SetObjectiveIcon("stop.tga");

    // wait for user to pull into Tinowle and stop
    Navigate.OnTrigger(me, trainShunter, "TrigStopFreight", Navigate.TRIGGER_STOPPED);
    trainShunter.SetAutopilotMode(Train.CONTROL_SCRIPT);
    trainShunter.SetDCCThrottle(0);
    trainShunter.SetHandBrake(true);

    // 400 points for arriving at Tilnowle
    Interface.AdjustScore(400);
    Interface.Print(stringTable.GetString1("aus_objective_arrive_tilnowle", "400"));
    Interface.Log(stringTable.GetString1("aus_objective_arrive_tilnowle", "400"));
    Sleep(3.0);


    // finish off the scenario
    Interface.SetObjective(stringTable.GetString1("scenario_complete_objective", Interface.GetScore()), "tick.tga");
    Interface.SetResults();
    Interface.LogResult(stringTable.GetString1("scenario_complete", Interface.GetScore()));
    Interface.Print(stringTable.GetString1("scenario_complete", Interface.GetScore()));
    scenarioDone = true;
    World.EndScenario(8.0);
  }

};