//
// BritishPassenger.gs
//
//  Copyright (C) 2002 Auran Developments Pty Ltd
//  All Rights Reserved.
//

include "trainz.gs"
include "common.gs"
include "world1.gs"
include "timetablescheduler.gs"
include "browser.gs"
include "stringtable.gs"


//
// Custom commands for AI passenger schedules
//

// Changes junctions so user train can go on to Teneriffe after Second AI HST has passed through
class AiPass2Start isclass CustomCommand
{
  bool Execute(Train train, int px, int py, int pz)
  {
    Navigate.LockJunction("Teneriffe", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Teneriffe 03", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Teneriffe Home", Junction.DIRECTION_RIGHT, false);
    return true;
  }
};

// Sends message to train to blast it's horn
class AiHornMsg isclass CustomCommand
{
  bool Execute(Train train, int px, int py, int pz)
  {
    train.Sleep(Math.Rand(px, py));
    train.SendMessage(train, "loco", "horn");
    return true;
  }
};


//
// class BritishPassenger
// brief This is the scenario class.
//
game class MyBritishPassenger isclass Scenario, ScenarioSupport
{
  bool scenarioDone = false;
  Train userTrain, aiPass1, aiPass2, aiPass3, aiFreight, aiDeltic;
  Schedule aiFreightSchedule, aiPass1BanksHeath, aiPass1Schedule, aiPass2Schedule, aiDelticPassSchedule;
  AiPass2Start aiPass2Start;
  AiHornMsg aiHorn;

  Signal signal1, signal2, signal3, signal4, hSig1, hSig2;  // signals that are specially scripted
  Signal stationSig1, stationSig2, stationSig3, stationSig4, stationSig5, stationSig6;  // station signals
  StationBehaviour stationBehaviour;

  StringTable stringTable;  // scenario string table


  //
  // Load will be called by Trainz to load the scenario map, and when the user presses Ctrl-L
  // param data is the save game data if loading a saved game.
  //
  bool Load(string data)
  {
    if(data and data.size())
    {
      Interface.Load(data);
    }

    // initialize scenario string table
    stringTable = ScenarioStringTable.Get();

    // load the map
    if(!World.LoadMap(World.FindKUID("britain")))
    {
      Interface.Log(stringTable.GetString1("std_error_loading_map_name", "britain"));
      return false;
    }

    // disable saving on this scenario
    BrowserInterface.SetSaveAllowed(false);


    //
    // Create train consists
    //

    // passenger train that user drives
    userTrain = World1.CreateTrain("userclass37passbr", "START", true);
//    userTrain = World1.CreateTrain("userclass37passbr", "Temp2", true); // debug/test
//    userTrain = World1.CreateTrain("userclass37passbr", "Start HST2", true); // debug/test
    userTrain.SetFloatingLimit(10.0 * Train.KPH_TO_MPS, true);

    // HST that rolls into Banks Heath at start
    aiPass1 = World1.CreateTrain("brhst1", "Start HST1", true);

    // HST that sits at teneriffe (pretends to be the HST that passes user at start!)
    aiPass2 = World1.CreateTrain("brhst1", "Start HST3", true);

    // HST that passes user on approach to teneriffe
    aiPass3 = World1.CreateTrain("brhst2", "Start HST4", true);

    // Freight that passes user slowly
    aiFreight = World1.CreateTrain("class37freight", "Start Freight", true);


    //
    // AI train schedules
    //

    // Class37 freight schedule (passes user in yard between Sutton Bridge and Hammond)
    aiFreightSchedule = new Schedule();
    aiFreightSchedule.StopAtTrigger("Trig StopFreight", 1);
    aiFreightSchedule.DeleteTrain();

    // AI HST that pulls into Banks Heath at start
    aiPass1BanksHeath = new Schedule();
    aiPass1BanksHeath.StopAtTrigger("TrigStopHSTBH", 1);
    aiPass1BanksHeath.Announce("HSTArrivedBanksHeath");

    // AI HST Passenger train 1 schedule (passes user at yard between Sutton Bridge and Hammond)
    aiPass1Schedule = new Schedule();
    aiHorn = new AiHornMsg();
    aiPass1Schedule.TakeJunction("Main Yard Nth", Junction.DIRECTION_LEFT);
    aiPass1Schedule.TakeJunction("Main Yard Nth1", Junction.DIRECTION_RIGHT);
    aiPass1Schedule.OnTrigger("TrigHST1Horn", true);
    aiPass1Schedule.Custom(aiHorn, 0, 2, 0);
    aiPass1Schedule.OnTrigger("TrigHST1Sig", true);
    aiPass1Schedule.SetSignal("S1", Signal.RED, "Blocking off line from this track.");
    aiPass1Schedule.TakeJunction("Main Yard Sth", Junction.DIRECTION_BACKWARD);
    aiPass1Schedule.TakeJunction("North Hammond", Junction.DIRECTION_BACKWARD);
    aiPass1Schedule.OnTrigger("Trig KillHST2", true);
    aiPass1Schedule.Announce("HST1Clear");
    aiPass1Schedule.DeleteTrain();

    // AI HST Passenger train 2 schedule (passes user on their approach to Teneriffe)
    aiPass2Schedule = new Schedule();
    aiPass2Start = new AiPass2Start();
    aiPass2Schedule.Custom(aiPass2Start);
    aiPass2Schedule.TakeJunction("Teneriffe 03", Junction.DIRECTION_BACKWARD);
    aiPass2Schedule.TakeJunction("Teneriffe", Junction.DIRECTION_BACKWARD);
    aiPass2Schedule.TakeJunction("Teneriffe Yard", Junction.DIRECTION_RIGHT);
    aiPass2Schedule.TakeJunction("Teneriffe Sth2", Junction.DIRECTION_LEFT);
    aiPass2Schedule.OnTrigger("TrigHST2Horn", true);
    aiPass1Schedule.Custom(aiHorn, 0, 0, 0);
    aiPass2Schedule.OnTrigger("Trig HSTClear", true);
    aiPass2Schedule.Announce("HST2Clear");
    aiPass2Schedule.TakeJunction("Teneriffe Home", Junction.DIRECTION_BACKWARD);
    aiPass2Schedule.OnTrigger("Trig KillHST3", true);
    aiPass2Schedule.DeleteTrain();

    // Deltic passenger schedule (passes user near between Tinoats & Bright Water)
    aiDelticPassSchedule = new Schedule();
    aiDelticPassSchedule.OnTrigger("Trig KillDelticPass", true);
    aiDelticPassSchedule.DeleteTrain();


    //
    // Create station behaviour for the user controlled passenger train. 
    // Note that scoring for station stuff is handled by the timetable scheduler code elsewhere, not by
    // the station behaviour except for the penalities.
    //
    stationBehaviour = new StationBehaviour();
    stationBehaviour.SetDefaults();
    stationBehaviour.warning.objective = "";
    stationBehaviour.warning.icon = "pause.tga";
    stationBehaviour.stopped.objective = "";
    stationBehaviour.stopped.icon = "stop.tga";
    stationBehaviour.stopped.wav = "bell.wav";
    stationBehaviour.stopped.score = 0;
    stationBehaviour.done.objective = "";
    stationBehaviour.done.icon = "pause.tga";
    stationBehaviour.done.score = 0;
    stationBehaviour.overrun.icon = "bad.tga";
    stationBehaviour.overrun.score = -300;
    stationBehaviour.reverse.icon = "bad.tga";
    stationBehaviour.mwstopped.icon = "bad.tga";
    stationBehaviour.mwstopped.alert = stringTable.GetString("std_bad_guard_clearence");
    stationBehaviour.mwstopped.colour = Interface.Colour_Red;
    stationBehaviour.mwstopped.score = -400;


    //
    // Set Junctions 
    //

    // Sutton Bridge station
    Navigate.LockJunction("Sutton Bridge", Junction.DIRECTION_RIGHT, false);

    // main yard junctions
    Navigate.LockJunction("Main Yard Nth", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Down Main Nth", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Main Yard Sth1", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Main Yard Sth", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("North Hammond", Junction.DIRECTION_RIGHT, false);

    // tinoats and brightwater station junctions
    Navigate.LockJunction("Tinoats", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Brightwater", Junction.DIRECTION_RIGHT, false);

    // Teneriffe junctions
    Navigate.LockJunction("Teneriffe Home", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Teneriffe Sth1", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Teneriffe Sth2", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Teneriffe Yard", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Teneriffe 02", Junction.DIRECTION_RIGHT, false);


    // 
    // Signals
    //

    // signals that need to be scripted especially to overcome problems
    signal1 = cast<Signal> Router.GetGameObject("S1");
    signal2 = cast<Signal> Router.GetGameObject("S2");
    signal3 = cast<Signal> Router.GetGameObject("S3");
    signal4 = cast<Signal> Router.GetGameObject("S4");

    // signals at stations (synchronised with timetable)
    stationSig1 = cast<Signal> Router.GetGameObject("SS1");
    stationSig2 = cast<Signal> Router.GetGameObject("SS2");
    stationSig3 = cast<Signal> Router.GetGameObject("SS3");
    stationSig4 = cast<Signal> Router.GetGameObject("SS4");
    stationSig5 = cast<Signal> Router.GetGameObject("SS5");
    stationSig6 = cast<Signal> Router.GetGameObject("SS6");

    stationSig1.SetSignalState(Signal.RED, "Wait until signal clears!");

    // signals used to get HST through (needed as a hardcoded work-around)
    hSig1 = cast<Signal> Router.GetGameObject("H1");
    hSig2 = cast<Signal> Router.GetGameObject("H2");

    // setup signals so if user passes one, scenario is terminated
    AddHandler(userTrain, "Train", "Entered red signal", "PassSignal");


    return true;
  }


  //
  // Save will be called by Trainz when the user presses Ctrl-S.
  // return the save game string, such that load will be able to restor the save game
  // from the last save check point.
  //  
  string Save()
  {
    return Interface.Save();
  }


  //
  // TrainDerailed will be called by Trainz when a train derails
  //
  void TrainDerailed(int trainId) 
  {
    if(!scenarioDone)
    {
      Interface.SetObjective(stringTable.GetString("std_terminated_derailed_train"), "bad.tga");
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);
      scenarioDone = true;
    }
  }


  //
  // Error message and termination method for when user crosses a trigger that they shouldn't have.
  // In this scenario, triggers are assigned to this method as user procedes on though the route, so
  // they can't backtrack too far (i.e. can't get back to a previous station).
  //
  void OffLimits(Message msg)
  {
    // verify it was the user's passenger train
    if (msg.src == userTrain)
    {
      Interface.SetObjective(stringTable.GetString("std_bad_backwards"), "bad.tga");
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);
    }
  }


  //
  // TrainCollided() will be called by Trainz when a train collides. A collision is rather bad, so
  // terminate the scenario.
  //
  void TrainCollided(int trainId) 
  {
    if(!scenarioDone)
    {
      Interface.SetObjective(stringTable.GetString("std_terminated_train_collision"), "bad.tga");       
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);
      scenarioDone = true;
    }
  }


  //
  // TrainSpeedingFine() is called by Trainz every second your trains speed exceeds the floating limit
  //
  void TrainSpeedingFine()
  {
    Interface.AdjustScore(-1);  
  }


  //
  // TrainBadCouple() is called by Trainz when vehicles couple greater than 8kph.
  //
  // Note that this shouldn't be a problem as no coupling is possible for the scenario.
  //
  void TrainBadCouple(int vehicleId)
  {
    Interface.SetObjective(stringTable.GetString("std_bad_couple_imperial"), "bad.tga");
    World.Play2DSound("ALARM.wav");
    Interface.AdjustScore(-50);
  }


  //
  // Called when train passes a red signal.
  //
  void PassSignal(Message msg)
  {
    Interface.SetObjective(stringTable.GetString("std_bad_redsignal"), "bad.tga");
    World.Play2DSound("ALARM.wav");
    World.EndScenario(5);
  }


  //
  // Handles the stopping of a train at a station including the station behvaiour, timetable and signal.
  // This method is simply a shortcut for repetetive code used for each station stop.
  //
  void DoStationStop(Signal sig, string name, string warn, string stop, string overrun, 
                     int depHour, int depMin, int depSec)
  {
    // Signal at end of station set to red (user passes it, scenario terminates)
    sig.SetSignalState(Signal.RED, "Wait until signal clears!");

    // navigate in to the station
    Navigate.Station(me, userTrain, name, warn, stop, overrun, null, stationBehaviour, 15, 20);

    // Guard's whistle, small random delay and clearence bell (along with play icon).
    World.Play2DSound("all_aboard.wav");
    Sleep(Math.Rand(4, 7));

    // Play clearence bell and update icon.
    // Note that just because the guard says it's clear, that doesn't mean the line is clear - guard
    // handles passenger safety, drivers job to look out for signals!
    World.Play2DSound("Buzzer.wav");
    Interface.SetObjectiveIcon("play.tga");

    // Wait until scehduled departure time and apply score. If late, a penalty will be applied and
    // the method will return immediately.
    TimetableScheduler.WaitUntilTime(depHour, depMin, depSec, me, true, 200);

    // Change signal to green so user can pass (at or after the scheduled time).
    sig.SetSignalState(Signal.GREEN, "Line clear.");

    // Ensure user can't go back down the track, scenario will terminate if they try as the OffLimits()
    // will be called.
    AddHandler(Router.GetGameObject(warn), "Object", "Enter", "OffLimits"); 
  }


  //
  // Main thread.
  //
  thread void main(void)
  {
    // Start the monitor thread to monitor speeding, derailing etc.
    Monitor();

    // Let user select DCC or Cabin Control fro the scenario
    SelectCabinDCC(userTrain);

    // Start off with 1000 points.  This will give us some points to play with before
    // the user obtains a negative score (if they do things wrong)
    Interface.AdjustScore(1000);


    //
    // Start - at Banks Heath station
    //

    // set time to 9:38
    World.SetGameTime(0.902);

    // user starts out with pause icon
    Interface.SetObjectiveIcon("pause.tga");

    // change cameras to HST
    World.SetCameraFlags(World.CAMERA_LOCKED);
    World.SetCamera(userTrain.GetFrontmostLocomotive(), World.CAMERA_TRACKING);

    // Setup HST to roll into Banks Heath and wait for it to stop
    aiPass1.RunSchedule(aiPass1BanksHeath, false, 0);
    Sleep(12);

    // Give control of camera back to user and place them in cab of their passenger train
    World.SetCameraFlags(World.CAMERA_NORMAL);
    World.SetCamera(userTrain.GetFrontmostLocomotive(), World.CAMERA_INTERNAL);

    // simulate movements of passengers around station for around a minute...
    Sleep(Math.Rand(2, 4));
    World.Play2DSound("plat_noise.wav"); 
    Sleep(Math.Rand(3, 5));

    // display cliboard and start thread so it will appear when user presses k
    DoInstructions();
    ManageBrowser();

    // Start at Banks Heath station, await guard's clearence to depart and leave the leave the station.
    World.Play2DSound("all_aboard.wav");
    Sleep(Math.Rand(18, 28));
    World.Play2DSound("Buzzer.wav");
    Interface.SetObjectiveIcon("play.tga"); // guard gives go ahead, but you must wait for signal!
    TimetableScheduler.WaitUntilTime(9, 41, 00, me, false, 0);
    stationSig1.SetSignalState(Signal.GREEN, "Line clear.");


    //
    // 1st station stop - Sutton Bridge
    //
    DoStationStop(stationSig2, "Sutton Bridge", "TrigWarn Sutton", "TrigStop Sutton", "TrigOver Sutton", 9, 46, 00);

    // When user departs Sutton Bridge, delete the HST at Banks Heath as it is no longer needed
    Navigate.OnTrigger(me, userTrain, "Trig KillHST1", Navigate.TRIGGER_ENTER);
    World.DeleteTrain(aiPass1);


    //
    // Freight and HST passings
    //

    // Trigger off the Class 37 freight and HST into their pass-by movements
    Navigate.OnTrigger(me, userTrain, "Trig StartFreightHST", Navigate.TRIGGER_ENTER);
    aiFreight.RunSchedule(aiFreightSchedule, false, 0);
    aiPass1 = World1.CreateTrain("BRHST1", "Start HST2", true);
    aiPass1.RunSchedule(aiPass1Schedule, false, 0);

    // setup signal for HST pass
    signal1.SetSignalState(Signal.GREEN, "HST Passing Through");
    stationSig3.SetSignalState(Signal.GREEN, "HST Passing Through");
    hSig1.SetSignalState(Signal.GREEN, "HST Passing Through");
    hSig2.SetSignalState(Signal.GREEN, "HST Passing Through");

    // Trigger off HST pass warning
    Navigate.OnTrigger(me, userTrain, "Trig WarnHST1", Navigate.TRIGGER_ENTER);
    Interface.SetObjectiveIcon("stop.tga");

    // Wait for HST to pass and clear the line (set signal to green for user)
    Timetable.OnAnnouncement(me, "HST1Clear");
    Navigate.LockJunction("Main Yard Sth", Junction.DIRECTION_LEFT, false);
    signal2.SetSignalState(Signal.YELLOW, "Letting passenger commuter go on.");
    hSig1.SetSignalState(Signal.AUTOMATIC, "");
    hSig2.SetSignalState(Signal.AUTOMATIC, "");

    Interface.SetObjectiveIcon("play.tga");
    AddHandler(Router.GetGameObject("Trig StartFreightHST"), "Object", "Enter", "OffLimits"); // ensure user can't go back down the track

    // 200 points for this passing
    Interface.AdjustScore(200);
    Interface.Print(stringTable.GetString2("british_HST_passing", "1st", "200"));
    Interface.Log(stringTable.GetString2("british_HST_passing", "1st", "200"));


    //
    // 2nd station stop - Hammond
    //
    DoStationStop(stationSig3, "Hammond", "TrigWarn Hammond", "TrigStop Hammond", "TrigOver Hammond", 10, 01, 30);


    //
    // 3rd station stop - Tinoats
    //
    DoStationStop(stationSig4, "Tinoats", "TrigWarn Tinoats", "TrigStop Tinoats", "TrigOver Tinoats", 10, 05, 0);

    // after Tinoats stop, we make a Deltic passenger train pass by
    aiDeltic = World1.CreateTrain("delticpass", "Start DelticPass", true);
    aiDeltic.RunSchedule(aiDelticPassSchedule, false, 0);
    AddHandler(Router.GetGameObject("TrigWarn Tinoats"), "Object", "Enter", "OffLimits"); // ensure user can't go back down the track


    //
    // 4th station stop - Bright Water
    //
    DoStationStop(stationSig5, "Bright Water", "TrigWarn Bright", "TrigStop Bright", "TrigOver Bright", 10, 09, 0);


    //
    // 5th station stop - Warradale
    //
    DoStationStop(stationSig6, "Warradale", "TrigWarn Warra", "TrigStop Warra", "TrigOver Warra", 10, 14, 0);


    //
    // HST pass thourgh before Teneriffe
    //

    // prepare signals
    signal3.SetSignalState(Signal.YELLOW, "HST Passby warning");
    signal4.SetSignalState(Signal.RED, "HST occupies this line!");

    // Trigger the 2nd HST
    Navigate.OnTrigger(me, userTrain, "Trig StartHST2", Navigate.TRIGGER_ENTER);
    aiPass3.RunSchedule(aiPass2Schedule, false, 0);
    
    // Warn user of passing HST
    Navigate.OnTrigger(me, userTrain, "Trig WarnHST2", Navigate.TRIGGER_ENTER);
    Interface.SetObjectiveIcon("stop.tga");

    // wait for HST to pass by through
    Timetable.OnAnnouncement(me, "HST2Clear");

    // change junctions and icon so user can procede on to Teneriffe
    Navigate.LockJunction("Teneriffe", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Teneriffe Sth2", Junction.DIRECTION_RIGHT, false);
    Interface.SetObjectiveIcon("play.tga");

    // 100 points for this passing
    Interface.AdjustScore(100);
    Interface.Print(stringTable.GetString2("british_HST_passing", "2nd", "100"));
    Interface.Log(stringTable.GetString2("british_HST_passing", "2nd", "100"));


    //
    // Final station - Tenneriffe
    //

    // modify station behaviour for termination
    stationBehaviour.stopped.objective = "";
    stationBehaviour.stopped.wav = ""; // redundant - already done in behaviour initialisation? was bell.wav
    stationBehaviour.stopped.icon = "tick.tga";

    // final station stop at Teneriffe (end of line)
    Navigate.Station(me, userTrain, "Teneriffe", "TrigWarn Tennrf", "TrigStop Tennrf", "TrigOver Tennrf", null, stationBehaviour, 5, 10);
    TimetableScheduler.WaitUntilTerminate(10, 24, 00, me, true, 300);
    Sleep(Math.Rand(4,5));

    // finish off the scenario
    Interface.SetObjective(stringTable.GetString1("scenario_complete_objective", Interface.GetScore()), "tick.tga");
    Interface.SetResults();
    Interface.LogResult(stringTable.GetString1("scenario_complete", Interface.GetScore()));
    Interface.Print(stringTable.GetString1("scenario_complete", Interface.GetScore()));
    scenarioDone = true;
    World.EndScenario(8.0);
  }
};

