//
// timetablescheduler.gs
//
//  Copyright (C) 2002 Auran Developments Pty Ltd
//  All Rights Reserved.
//

include "trainz.gs"
include "stringtable.gs"


//
// Static class that provides some basic timetable scheduling functionality.
//
static class TimetableScheduler
{
	StringTable stringTable = ScenarioStringTable.Get(); // scenario string table

	// Declarations
	public void WaitUntilTime(int hours, int minutes, int seconds, GameObject go, bool doScoring, int ontimeScore);
	public void WaitUntilTerminate(int hours, int minutes, int seconds, GameObject go, bool doScoring, int ontimeScore);
	public void WaitUntilGameTime(float gameTime, GameObject go);
	public void WaitForNextDay(GameObject go);
	public string TimeToString(int hours, int minutes);


	// Implementation

	// 
	// wait until specified realworld time value
	// returns immediately if the specified time is already passed in the current day
	//
	public void WaitUntilTime(int hours, int minutes, int seconds, GameObject go, bool doScoring, int ontimeScore)
	{
		int temp;
		float gametime = 0;
        string num1;

		// calculate game time as a normalized value
		gametime = hours;
		gametime = gametime + minutes / 60.0;
		gametime = gametime + seconds / 3600.0;
		gametime = gametime / 24.0;
		gametime = gametime + 0.5;
		if (gametime > 1.0)
			gametime = gametime - 1.0;

		// calculate difference between game time and timetable time
		float diffgameType = gametime - World.GetGameTime();
		diffgameType = diffgameType * (24.0 * 3600.0);
		float offset = gametime - World.GetGameTime();

		if (doScoring)
		{
			// early
			if (diffgameType >= 30.0)
			{
				num1 = (int)diffgameType;

				Interface.Print(stringTable.GetString2("std_timetable_early_depart", num1, TimeToString(hours, minutes)));
				Interface.Print(stringTable.GetString1("std_timetable_ontime_alt", ontimeScore));
				Interface.Log(stringTable.GetString2("std_timetable_early_depart", num1, TimeToString(hours, minutes)));
				Interface.Log(stringTable.GetString1("std_timetable_ontime_alt", ontimeScore));
				Interface.AdjustScore(ontimeScore);  
			}
            // late
			else if (diffgameType <= -30.0)
			{
                temp = Math.Abs((int)diffgameType);

				Interface.Print(stringTable.GetString2("std_timetable_late", temp, (temp-30)));
				Interface.Log(stringTable.GetString2("std_timetable_late", temp, (temp-30)));
				Interface.AdjustScore((-temp)-30);
			}
			// ontime (note quite Swiss or Japanese timing, we allow a 30 second tolerance level!)
            else
            {
				Interface.Print(stringTable.GetString1("std_timetable_ontime", ontimeScore));
				Interface.Log(stringTable.GetString1("std_timetable_ontime", ontimeScore));
				Interface.AdjustScore(ontimeScore);
			}
		}
		WaitUntilGameTime(gametime, go);
	}


	//
	// Variation of WaitUntilTime for terminating at the end of the line (i.e. it doesn't wait around).
	// Calculates score etc and jumps straight out.
	//
	public void WaitUntilTerminate(int hours, int minutes, int seconds, GameObject go, bool doScoring, int ontimeScore)
	{
		int temp;
		float gametime = 0;
        string num1;

		// calculate game time as a normalized value
		gametime = hours;
		gametime = gametime + minutes / 60.0;
		gametime = gametime + seconds / 3600.0;
		gametime = gametime / 24.0;
		gametime = gametime + 0.5;
		if (gametime > 1.0)
			gametime = gametime - 1.0;

		// calculate difference between game time and timetable time
		float diffgameType = gametime - World.GetGameTime();
		diffgameType = diffgameType * (24.0 * 3600.0);
		float offset = gametime - World.GetGameTime();

		// do scoring (if specified)
		if (doScoring)
		{
			// early
			if (diffgameType >= 30.0)
			{
				num1 = (int)diffgameType;

				Interface.Print(stringTable.GetString2("std_timetable_early", num1, TimeToString(hours, minutes)));
				Interface.Print(stringTable.GetString1("std_timetable_ontime_alt", ontimeScore));
				Interface.Log(stringTable.GetString2("std_timetable_early", num1, TimeToString(hours, minutes)));
				Interface.Log(stringTable.GetString1("std_timetable_ontime_alt", ontimeScore));
				Interface.AdjustScore(ontimeScore);
			}
            // late
			else if (diffgameType <= -30.0)
			{
                temp = Math.Abs((int)diffgameType);

				Interface.Print(stringTable.GetString2("std_timetable_late", temp, (temp-30)));
				Interface.Log(stringTable.GetString2("std_timetable_late", temp, (temp-30)));
				Interface.AdjustScore((-temp)-30);
			}
			// ontime (note quite Swiss or Japanese timing, we allow a 30 second tolerance level!)
            else
            {
				Interface.Print(stringTable.GetString1("std_timetable_ontime", ontimeScore));
				Interface.Log(stringTable.GetString1("std_timetable_ontime", ontimeScore));
				Interface.AdjustScore(ontimeScore);
			}
		}
	}


	// 
	// wait until specified time
	// returns immediately if the specified time is already passed in the current day
	//
	public void WaitUntilGameTime(float gameTime, GameObject go)
	{
		while (World.GetGameTime() < gameTime)
		{
			// ideally we would pick a more accurate (longer) time interval here
			// however this could get a little difficult since some other fellow could change the time on us
			// (condition variables anyone? :)
			go.Sleep(1.0);
		}
	}


	// 
	// waits until the next day
	//
	public void WaitForNextDay(GameObject go)
	{
		float startGameTime = World.GetGameTime();
		
		// if we happen to be really, really close to the start of the day,
		// it's unlikely that we'll ever get out of this function by polling every one second
		// so lets pretend that this is the start of a nice new day (as if they had called just before midnight; about a second ago)
		if (startGameTime <= 1.5 / (24.0 * 3600.0))
			return;
		
		while (World.GetGameTime() >= startGameTime)
			// see above for a discussion of why this isn't ideal
			go.Sleep(1.0);
	}


    //
    // Converts the given time in hours and minutes into a digital watch-like display which is 
	// returned as a string.
	//
    public string TimeToString(int hours, int minutes)
    {
      string returnTime;

      if (minutes < 10)
        returnTime = (hours + ":0" + minutes);
      else
        returnTime = (hours + ":" + minutes);

	  return returnTime;
    }
};
