//
// HighValMine.gs
//
//  Copyright (C) 2002 Auran Developments Pty Ltd
//  All Rights Reserved.
//

include "trainz.gs"
include "common.gs"
include "world1.gs"
include "stringtable.gs"


//
// Custom commands for AI coal train
//

// Sends message to train to blast it's horn
class AiHornMsg isclass CustomCommand
{
  bool Execute(Train train, int px, int py, int pz)
  {
    train.Sleep(Math.Rand(px, py));
    train.SendMessage(train, "loco", "horn");
    return true;
  }
};


//
// Structure for storing point data of an objective.
//
class ScoreData
{
  public string objName;  // objective name
  public int objective;   // points for this objective
  public int junc;        // junction changes done
  public int couple;      // couple operations done
  public int decouple;    // decouple operations done
  public int score;       // final score (calculated from above 4 values: 
                          //   score = objective - 5 * (junc + couple + decouple)
};


//
// class MyHighValMine
// brief This is the scenario class.
//
game class MyHighValMine isclass Scenario, ScenarioSupport
{
  bool scenarioDone = false;
  bool recentBadCouple = false;
  int objCount = 0;
  int speedingDeduction = 0;  //
  int badCouplings = 0;       // keep track of bad couplings user has done for final score
  ScoreData[] scoreData;      // array of accumulated score data (most objectives have a slot in this array)
  string objectiveIcon = null;

  Train userTrain, boxcarStart, coalCars1, coalCars2, aiInner, aiOuter, pickupF7, robbinsBoxCar, robbinsCoalCar;
  Vehicle loco, greenBoxCarVehicle, robbinsBoxCarVehicle;
  ShuntTrack[] assortedCars;
  Schedule initialPassSchedule, innerMainLineSchedule, outerMainLineSchedule, stormontAiPassSchedule;
  AiHornMsg aiHorn;
  GameObject coalParticle1, coalParticle2;  // coal particle emitters at Greenwood coal dump

  StringTable stringTable;  // scenario string table


  //
  // Load will be called by Trainz to load the scenario map, and when the user presses Ctrl-L
  // param data is the save game data if loading a saved game.
  //
  bool Load(string data)
  {
    int i;
    Vehicle[] tempVehicleArray;

    if(data and data.size())
    {
      Interface.Load(data);
    }

    // initialize scenario string table
    stringTable = ScenarioStringTable.Get();

    // load the map
    if(!World.LoadMap(World.FindKUID("highlandvalley")))
    {
      Interface.Log(stringTable.GetString1("std_error_loading_map_name", "highlandvalley"));
      return false;
    }

    // disable saving on this scenario
    BrowserInterface.SetSaveAllowed(false);

    // try and gain pointer to coal dust particles
    coalParticle1 = Router.GetGameObject("Coal1");
    coalParticle2 = Router.GetGameObject("Coal2");
    if (!coalParticle1 or !coalParticle2)
    {
      Interface.Log(stringTable.GetString1("std_error_loading_particleFX", "Coal Dust"));
      return false;
    }


    // Initialize scoring data array. This array is gradually filled with data as user progresses
    // through scenario for each objective that requires switching so a detailed scorecard can be
    // produced at the end of the scenario.
    scoreData = new ScoreData[7]; // 7 objectives to track statistics on
    for (i = 0; i < 7; i++)
    {
      scoreData[i] = new ScoreData();
      scoreData[i].junc = scoreData[i].couple = scoreData[i].decouple = scoreData[i].score = 0;
    }

    // set objective names and completion points for each one
    scoreData[0].objName   = "Greenwood Yard";
    scoreData[0].objective = 400;
    scoreData[1].objName   = "Robbins Pickup";
    scoreData[1].objective = 300;
    scoreData[2].objName   = "Odlum Mine Drop";
    scoreData[2].objective = 500;
    scoreData[3].objName   = "Afton Mine Drop";
    scoreData[3].objective = 500;
    scoreData[4].objName   = "Highland Valley Drop";
    scoreData[4].objective = 200;
    scoreData[5].objName   = "Afton Mine Pickup";
    scoreData[5].objective = 400;
    scoreData[6].objName   = "Odlum Mine Pickup";
    scoreData[6].objective = 400;


    //
    // Create train consists
    //

    // user's loco in roundhouse
    userTrain = World1.CreateTrain("F71", "Stall 3", false);
//    userTrain = World1.CreateTrain("TempB4Robbins", "TempB4Robbins", true); // test/debug
//    userTrain = World1.CreateTrain("TempB4Robbins", "ROBBINSCROSS", false); // test/debug
//    userTrain = World1.CreateTrain("TempB4Stormont", "TempB4Stormont", true); // test/debug
//    userTrain = World1.CreateTrain("TempB4Stormont", "Stormont Cross", true); // test/debug
//    userTrain = World1.CreateTrain("TempB4Afton", "Wyenot Consist", true); // test/debug
//    userTrain = World1.CreateTrain("TempB4HV", "StartB4HV", true); // test/debug
//    userTrain = World1.CreateTrain("F71", "StartB4HV", false); // test/debug
//    userTrain = World1.CreateTrain("F71", "Wyenot Consist", false); // test/debug
//    userTrain = World1.CreateTrain("TempB4Stormont", "TempB4Robbins", true); // test/debug
    userTrain.SetFloatingLimit(10.0 * Train.KPH_TO_MPS, true);  // floating limit of ~ 6mph
    userTrain.SetAutopilotMode(Train.CONTROL_SCRIPT); // disable user control until we are ready to roll later on
    loco = userTrain.GetFrontmostLocomotive();        // get vehicle pointer to our loco

    // empty coal consists and boxcar in the yard
    coalCars1   = World1.CreateTrain("6EmptyCoalCars", "Class 2", true);
    coalCars2   = World1.CreateTrain("4EmptyCoalCars", "PickupCoal2", true);
    boxcarStart = World1.CreateTrain("BoxCarSmall", "StartGWBoxCar", false);

    // initial ai passenger train
    aiInner = World1.CreateTrain("LongPass", "StartInnerAI", true);

    // create box car and 2 coal cars in Robbins for user to pickup
    robbinsBoxCar  = World1.CreateTrain("BoxCarBig", "RobbinsPickup2", true);
    robbinsCoalCar = World1.CreateTrain("2EmptyCoalCars", "Robbins Team", true);
    robbinsBoxCar.SetHandBrake(true);
    robbinsCoalCar.SetHandBrake(true);


    //
    // Tracking array
    //

    // allocate memory
    assortedCars = new ShuntTrack[14];
    for (i = 0; i < 14; i++)
    {
      assortedCars[i] = new ShuntTrack();
      assortedCars[i].name = "Coal";
      assortedCars[i].trigger = null;
    }

    // first lot of coal cars in Greenwood yard (6)
    tempVehicleArray = coalCars1.GetVehicles();
    for (i = 0; i < 6; i++)
      assortedCars[i].vehicle = tempVehicleArray[i];

    // second lot of coal cars in Greenwood yard (4)
    tempVehicleArray = coalCars2.GetVehicles();
    for (i = 6; i < 10; i++)
      assortedCars[i].vehicle = tempVehicleArray[i-6];

    // coal cars in Robbins
    tempVehicleArray = robbinsCoalCar.GetVehicles();
    assortedCars[10].vehicle = tempVehicleArray[0];
    assortedCars[11].vehicle = tempVehicleArray[1];

    // box car that starts in Greenwood yard
    tempVehicleArray = boxcarStart.GetVehicles();
    assortedCars[12].vehicle = tempVehicleArray[0];
    assortedCars[12].name = "SmallBoxCar";

    // box car picked up at Robbins
    tempVehicleArray = robbinsBoxCar.GetVehicles();
    assortedCars[13].vehicle = tempVehicleArray[0];
    assortedCars[13].name = "BigBoxCar";


    //
    // Schedules
    //

    // schedule of AI passenger train that stops in station and moves on
    initialPassSchedule = new Schedule();
    aiHorn = new AiHornMsg();
    initialPassSchedule.OnTrigger("TrigInnerHorn1", true);
    initialPassSchedule.Custom(aiHorn, 2, 4, 0);
    initialPassSchedule.OnTrigger("GPE", true);
    initialPassSchedule.StopTrain(Math.Rand(140, 180));
    initialPassSchedule.OnTrigger("TrigInnerHorn2", true);
    initialPassSchedule.Custom(aiHorn, 2, 4, 0);
    initialPassSchedule.OnTrigger("TrigKillInnerAI", true);
    initialPassSchedule.Announce("InitialPassScheduleDone");
    initialPassSchedule.DeleteTrain();

    // schedule for passenger train that passes user at Stormont
    stormontAiPassSchedule = new Schedule();
    stormontAiPassSchedule.OnTrigger("SPEEDHERE", true);
    stormontAiPassSchedule.Custom(aiHorn, 2, 4, 0);
    stormontAiPassSchedule.OnTrigger("SPEEDHERE", false);
    stormontAiPassSchedule.Announce("StormontPassClear");
    stormontAiPassSchedule.OnTrigger("StormontStop", true);
    stormontAiPassSchedule.StopTrain(120);
    stormontAiPassSchedule.OnAnnounce("KillStormontPass");
    stormontAiPassSchedule.DeleteTrain();

    // inner main line AI schedule
    innerMainLineSchedule = new Schedule();
    innerMainLineSchedule.OnTrigger("LINECLEAR", true);
    innerMainLineSchedule.OnTrigger("TrigKillInnerAI", true);
    innerMainLineSchedule.Announce("MainLineScheduleDone");
    innerMainLineSchedule.DeleteTrain();

    // outer main line AI schedule
    outerMainLineSchedule = new Schedule();
    outerMainLineSchedule.OnTrigger("GREENWODD KILL TEAM", true);
    outerMainLineSchedule.OnTrigger("TrigKillOuterAI", true);
    outerMainLineSchedule.Announce("MainLineScheduleDone");
    outerMainLineSchedule.DeleteTrain();


    //
    // Junctions
    //

    // Greenwood Yard
    Navigate.LockJunction("Arr Dep North", Junction.DIRECTION_RIGHT, true);
    Navigate.LockJunction("Engine Main", Junction.DIRECTION_RIGHT, true);
    Navigate.LockJunction("X1ASW", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("X1ANW", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Cominco Loads North", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Comminco Mill", Junction.DIRECTION_LEFT, false);

    // Robbins
    Navigate.LockJunction("Robbins North", Junction.DIRECTION_RIGHT, true);
    Navigate.LockJunction("Robbins South", Junction.DIRECTION_LEFT, true);

    // Stormont
    Navigate.LockJunction("Stormont South", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Stormont North", Junction.DIRECTION_RIGHT, false);

    // Odlum Wye and mine
    Navigate.LockJunction("Odlum South", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Odlum North", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Odlum East", Junction.DIRECTION_LEFT, true);
    Navigate.LockJunction("Odlum 1", Junction.DIRECTION_RIGHT, true);
    Navigate.LockJunction("Odlum 2", Junction.DIRECTION_RIGHT, true);
    Navigate.LockJunction("Odlum3", Junction.DIRECTION_LEFT, true);
    Navigate.LockJunction("Odlum4", Junction.DIRECTION_LEFT, true);

    // Wyenot junction and Afton mine
    Navigate.LockJunction("Wyenot Jct", Junction.DIRECTION_RIGHT, true); // was false
    Navigate.LockJunction("Afton Wye South", Junction.DIRECTION_LEFT, true);
    Navigate.LockJunction("Afton South", Junction.DIRECTION_RIGHT, true);
    Navigate.LockJunction("Afton Wye West", Junction.DIRECTION_RIGHT, true);
    Navigate.LockJunction("Afton End", Junction.DIRECTION_LEFT, true);
    Navigate.LockJunction("AftTrack3", Junction.DIRECTION_RIGHT, true);

    // Closed circuit for double main line (AI trains run around this)
    Navigate.LockJunction("X1 SW", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("X1 NW", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("X1 SE", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("X1 NE", Junction.DIRECTION_LEFT, false);

    Navigate.LockJunction("X1ASW", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("X1ANW", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("X1ASE", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("X1ANE", Junction.DIRECTION_LEFT, false);

    Navigate.LockJunction("Greenwood Terminal", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Afton Branch", Junction.DIRECTION_RIGHT, false);

    Navigate.LockJunction("Greenwood Team Track", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Greenwood Yard South", Junction.DIRECTION_RIGHT, false);

    Navigate.LockJunction("X2 SW", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("X2 NW", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("X2 SE", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("X2 NE", Junction.DIRECTION_LEFT, false);

    Navigate.LockJunction("X3 SW", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("X3 NW", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("X3 SE", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("X3 NE", Junction.DIRECTION_LEFT, false);

    Navigate.LockJunction("Lumby Pass North", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Lumby Pass South", Junction.DIRECTION_RIGHT, false);

    Navigate.LockJunction("X4 SW", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("X4 NW", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("X4 SE", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("X4 NE", Junction.DIRECTION_LEFT, false);

    return true;
  }


  //
  // Save will be called by Trainz when the user presses Ctrl-S.
  // return the save game string, such that load will be able to restore the save game
  // from the last save check point.
  //  
  string Save()
  {
    return Interface.Save();
  }


  //
  // TrainDerailed() will be called by Trainz when a train derails
  //
  void TrainDerailed(int trainId) 
  {
    if(!scenarioDone)
    {
      Interface.SetObjective(stringTable.GetString("std_terminated_derailed_train"), "bad.tga");
      World.EndScenario(5);  
      scenarioDone = true;
    }
  }


  //
  // TrainCollided() will be called by Trainz when a train collides
  //
  void TrainCollided(int trainId) 
  {
    if(!scenarioDone)
    {
      Interface.SetObjective(stringTable.GetString("std_terminated_train_collision"), "bad.tga");       
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);  
      scenarioDone = true;
    }
  }


  //
  // TrainSpeedingFine() is called by Trainz every second your trains speed exceeds the floating limit
  //
  void TrainSpeedingFine() 
  {
    Interface.AdjustScore(-1);

    // increment speeding counter so we can give user total amount of points lost for speeding on 
    // score card at end
    speedingDeduction++;
  }


  //
  // Warns user of their bad couple-up and displays a message along with alarm sound
  //
  thread void BadCouple()
  {
    // set to true so this method won't be called repeatedly in near proximity
    recentBadCouple = true;

    // increment counter so amount of bad couplings can be displayed to user in score card
    badCouplings++;

    // warn user and deduct points
    Interface.SetObjective(stringTable.GetString("std_bad_couple_imperial"), "bad.tga");
    Interface.AdjustScore(-50);
    World.Play2DSound("ALARM.wav");

    // recentDecouple reset after delay so we don't get 2 BadCouple() calls too close to each other
    Sleep(1.0);
    recentBadCouple = false;

    // after brief sleep, hide warning message and restore previous icon
    Sleep(3.0);
    Interface.SetObjective("", objectiveIcon);
    Interface.ShowObjective(false);
  }


  //
  // TrainBadCouple() is called by Trainz when vehicles couple greater than 8kph (5mph).
  //
  void TrainBadCouple(int vehicleId)
  {
    // was it a recent bad couple? If so, display message and alarm
    if (!recentBadCouple) 
      BadCouple();
  }


  //
  // Returns time difference in seconds of given normalized time values
  // Note: It is expected that time2 is the latter!
  //
  float TimeDifference(float time1, float time2)
  {
    return (time2 - time1) * (24.0 * 3600.0);
  }


  //
  // Runs AI trains around the mainlines to provide dynamic scenery of interest 'down below' as 
  // the user makes their way up to Highland Valley and back.
  //
  thread void RunAITrains(void)
  {
    bool rand1, rand2, rand3;
    Schedule toRunSchedule;
    Train aiTrain;
    string startPosition, trainConsist;
  
    Interface.Log("Start of RunAITrains()"); // test/debug


    // wait until initial AI passenger train has finished before we start running AI trains
    Timetable.OnAnnouncement(me, "InitialPassScheduleDone");

    // loops until scenario is done, run a random train for each loop cycle
    while (!scenarioDone)
    {
      // generate some random bool values
      rand1 = TrainUtil.RandBool();
      rand2 = TrainUtil.RandBool();
      rand3 = TrainUtil.RandBool();

      // randomly select a track (inner or outer)
      if (rand1)
      {
        toRunSchedule = innerMainLineSchedule;
        startPosition = "StartInnerAI";
      }
      else
      {
        toRunSchedule = outerMainLineSchedule;
        startPosition = "StartOuterAI";
      }

      // randomly select one of 4 possible consists
      if ((rand2) and (rand3))
        trainConsist = "AlcoFreight";
      else if ((rand2) and (!rand3))
        trainConsist = "LoadedCoalFreight";
      else if ((!rand2) and (rand3))
        trainConsist = "LongMixedFreight";
      else  // ((!rand2) and (!rand3))
        trainConsist = "LongPass";

      // log train being run
      Interface.Log("Running an AI train that is " + trainConsist + ":" + startPosition);

      // create the train and run it's schedule
      aiTrain = World1.CreateTrain(trainConsist, startPosition, true);
      aiTrain.RunSchedule(toRunSchedule, false, 0);

      // wait for schedule to finish before we finish the loop cycle and run another train
      Timetable.OnAnnouncement(me, "MainLineScheduleDone");

      // Remove the announcement, otherwise it stays there on the timetable and OnAnnouncement()
      // call above would intercept the announcement before the next trains has finished its
      // run in the next cycle of this loop.
      Timetable.UnAnnounce("MainLineScheduleDone");

      // sleep a bit before another train is run
      Sleep(15);
    }

    Interface.Log("End of RunAITrains()"); // test/debug
  }


  //
  // Gather statistics on the current objective so scoring can be applied and compiled for display later.
  // Information gathered:
  //   - amount of couple-ups
  //   - amount of junction changes performed
  //   - amount of decouplings
  //
  thread void ScoringCounter()
  {
    int i;
    Message msg;
    Vehicle tempVehicle;
    float prevTimeStamp = 0.0f; //World.GetGameTime();
    float currentTime;

    Interface.Log("Start of ScoringCounter()"); // test/debug


    // wait until kill message is sent while processing message to modify counters
    wait ()
    {
      // every junction toggle increments counter
      on "Junction", "Toggled", msg :
      {
        scoreData[objCount].junc++;
        continue;
      }
      // If couple operation is on loco, increment decouple counter
      on "Vehicle", "Coupled", msg :
      {
        tempVehicle = cast<Vehicle> msg.src;
        if (tempVehicle.GetMyTrain() == loco.GetMyTrain())
        {
          currentTime = World.GetGameTime();
          if (TimeDifference(prevTimeStamp, currentTime) > 1.0)
          {
            prevTimeStamp = World.GetGameTime();
            scoreData[objCount].couple++;
            Interface.Log("Train couple-up detected!"); // temp/debug
          }
        }
        continue;
      }
      // If decouple operation is on loco, increment decouple counter
      on "Vehicle", "Decoupled", msg :
      {
        tempVehicle = cast<Vehicle> msg.src;
        if (tempVehicle)
          if (tempVehicle.GetMyTrain() == loco.GetMyTrain())
            scoreData[objCount].decouple++;
        continue;
      }
      // kill message has been sent, get out of here
      on "Kill", "SwitchingScore", msg:
      {
        break;
      }
    }

    Interface.Log("End of ScoringCounter()"); // test/debug
  }


  //
  // Load up HTML window on screen and display the clipboard instruction page
  //
  void DoScoreCard(void)
  {
    Message msg;
    string HTMLpage; // string that will contain HTML text - we are constructing score tables on the run and 
                     // a pre-defined HTML file wouldn't do.

    Browser scoreCard = BrowserInterface.CreateBrowser();


    // assemble HTML code from the string table
    HTMLpage = 
      // start of page header
      stringTable.GetString("hvc_html_head") +
      stringTable.GetString("hvc_html_title") +

      // points scored table
      stringTable.GetString("hvc_html_tablestart") +
      stringTable.GetString3("hvc_html_tablerow_3", "<b>Reason</b>", " ", "<b>Points</b>") +
      stringTable.GetString3("hvc_html_tablerow_3", "Starting Score", " ", " 1000") +
      stringTable.GetString3("hvc_html_tablerow_3", "Greenwood Yard", " ", scoreData[0].score) +
      stringTable.GetString3("hvc_html_tablerow_3", "Robbins Pickup", " ", scoreData[1].score) +
      stringTable.GetString3("hvc_html_tablerow_3", "Stormont Passing", " ", " 200") +
      stringTable.GetString3("hvc_html_tablerow_3", "Odlum Mine Drop", " ", scoreData[2].score) +
      stringTable.GetString3("hvc_html_tablerow_3", "Afton Mine Drop", " ", scoreData[3].score) +
      stringTable.GetString3("hvc_html_tablerow_3", "Highland Valley Drop", " ", scoreData[4].score) +
      stringTable.GetString3("hvc_html_tablerow_3", "Afton Mine Pickup", " ", scoreData[5].score) +
      stringTable.GetString3("hvc_html_tablerow_3", "Odlum Mine Pickup", " ", scoreData[5].score) +
      stringTable.GetString3("hvc_html_tablerow_3", "Coal Dump", " ", " 300") +
      stringTable.GetString3("hvc_html_tablerow_3", "Speeding", " ", "-" + speedingDeduction) +
      stringTable.GetString3("hvc_html_tablerow_3", "Bad Coupling", " ", "-" + (badCouplings * 50)) +
      stringTable.GetString3("hvc_html_tablerow_3", " ", "<b>Total:</b>", Interface.GetScore()) +
      stringTable.GetString("hvc_html_tableclose") +

      // detailed operation deductions table
      "<br><br><b>Deductions for Operations Performed:</b><br>" +
      stringTable.GetString("hvc_html_tablestart") +
      stringTable.GetString6("hvc_html_tablerow_6", "<b>Objective</b>", "<b>Points</b>", "<b>Junction<br>Change</b>",
                             "<b>Couple</b>", "<b>Decouple</b>", "<b>Deduction</b>") +
      stringTable.GetString6("hvc_html_tablerow_6", "Greenwood Yard", scoreData[0].objective, scoreData[0].junc, scoreData[0].couple, scoreData[0].decouple, "-" + (scoreData[0].objective - scoreData[0].score)) +
      stringTable.GetString6("hvc_html_tablerow_6", "Robbins Pickup", scoreData[1].objective, scoreData[1].junc, scoreData[1].couple, scoreData[1].decouple, "-" + (scoreData[1].objective - scoreData[1].score)) +
      stringTable.GetString6("hvc_html_tablerow_6", "Odlum Mine Drop", scoreData[2].objective, scoreData[2].junc, scoreData[2].couple, scoreData[2].decouple, "-" + scoreData[2].score) +
      stringTable.GetString6("hvc_html_tablerow_6", "Afton Mine Drop", scoreData[3].objective, scoreData[3].junc, scoreData[3].couple, scoreData[3].decouple, "-" + scoreData[3].score) +
      stringTable.GetString6("hvc_html_tablerow_6", "Highland Valley Drop", scoreData[4].objective,  scoreData[4].junc, scoreData[4].couple, scoreData[4].decouple, "-" + scoreData[4].score) +
      stringTable.GetString6("hvc_html_tablerow_6", "Afton Mine Pickup", scoreData[5].objective, scoreData[5].junc, scoreData[5].couple, scoreData[5].decouple, "-" + scoreData[5].score) +
      stringTable.GetString6("hvc_html_tablerow_6", "Odlum Mine Pickup", scoreData[6].objective, scoreData[6].junc, scoreData[6].couple, scoreData[6].decouple, "-" + scoreData[6].score) +
      stringTable.GetString("hvc_html_tableclose") +

      // close page
      stringTable.GetString("hvc_html_tail");

    // load HTML page in a window
    scoreCard.LoadHTMLString(HTMLpage);

    // wait for user to close window
    wait()
    {
      on "Browser-Closed", "", msg :
      {
        // fade back in...
        break;
      }
    }
  }

 
  // 
  // Groups scoring functionality into one neat method.
  //
  void UpdateScore(string objectiveDone, string nextObjective)
  {
    int total;

    // Calculate deduction for coupling, decoupling and junction change operations performed
    total = scoreData[objCount].junc + scoreData[objCount].couple + scoreData[objCount].decouple;
    scoreData[objCount].score = scoreData[objCount].objective - (total * 5);

    // Add points for completing the objective
    Interface.AdjustScore(scoreData[objCount].score);
    Interface.Print(stringTable.GetString1(objectiveDone, scoreData[objCount].score));
    Interface.Log(stringTable.GetString1(objectiveDone, scoreData[objCount].score));

    // small delay and next objective
    Sleep(5);
    Interface.Print(nextObjective);

    // increment objective counter 
    objCount++;
  }


  //
  // Handles the user for the start period where they are to get out of round house,
  // through turntable and pickup all coal cars ready for departure.
  //
  void StartGreenwood(void)
  {
    bool carsPickedUp = false;
    int  i, coalcarcount1, coalcarcount2, totalcarcount;
    Vehicle[] tempVehicleArr1, tempVehicleArr2;
    ShuntTrack[] tracking;

    Interface.Log("Start of StartGreenwood()"); // test/debug


    //
    // Conversation 1 before user takes off for the day...
    //

    // play the first conversation
    Sleep(Math.Rand(2,5));
    Sleep(World.Play2DSound("Conv1.wav"));

    // hand over control to user
    objectiveIcon = "play.tga";
    Interface.SetObjectiveIcon("play.tga");
    userTrain.SetAutopilotMode(Train.CONTROL_MANUAL);

    World.SetCameraFlags(World.CAMERA_NORMAL);
    World.SetCamera(loco, World.CAMERA_INTERNAL);

    // display instruction sheet and setup thread to trap keystroke so it can be called back again
    DoInstructions();
    ManageBrowser();


    //
    // Prepare vehicle tracking array
    //
    tempVehicleArr1 = coalCars1.GetVehicles();
    coalcarcount1 = tempVehicleArr1.size();
    tempVehicleArr2 = coalCars2.GetVehicles();
    coalcarcount2 = tempVehicleArr2.size();

    totalcarcount = coalcarcount1 + coalcarcount2;
    tracking = new ShuntTrack[totalcarcount+1];

    // first consist of coal cars
    for (i = 0; i < coalcarcount1; i++)
    {
      tracking[i] = new ShuntTrack();
      tracking[i].vehicle = tempVehicleArr1[i];
      tracking[i].trigger = null;
      tracking[i].name = null;
    }

    // second consist of coal cars
    for (i = coalcarcount1; i < totalcarcount; i++)
    {
      tracking[i] = new ShuntTrack();
      tracking[i].vehicle = tempVehicleArr2[i-coalcarcount1];
      tracking[i].trigger = null;
      tracking[i].name = null;
    }

    // lone boxcar
    tempVehicleArr1 = boxcarStart.GetVehicles();
    tracking[totalcarcount] = new ShuntTrack();
    tracking[totalcarcount].vehicle = tempVehicleArr1[0];
    tracking[totalcarcount].trigger = null;
    tracking[totalcarcount].name = null;


    //
    // Pickup cars
    //

    // start score counting thread
    ScoringCounter();

    // loop until user's loco is at head of consist containing all required cars
    while(!carsPickedUp)
    {
      // wait until all cars required are in a consist with the loco
      TrainUtil.WaitUntilHas(loco, tracking);

      // verify user consist has loco at head and only the cars needed (via length)
      tempVehicleArr1 = loco.GetMyTrain().GetVehicles();
      i = tempVehicleArr1.size();

      if ((loco == loco.GetMyTrain().GetFrontmostLocomotive()) and (i == (totalcarcount + 2)))
        carsPickedUp = true;
    }

    // update user's train pointer
    userTrain = loco.GetMyTrain();

    // kill off score counting thread
    SendMessage(me, "Kill", "SwitchingScore");

    // block off all coupling and decoupling operations once train is assembled
    userTrain.SetCouplingMask(Vehicle.COUPLE_NONE); 
    userTrain.SetDecouplingMask(Vehicle.COUPLE_NONE);

    Interface.Log("End of StartGreenwood()"); // test/debug
  }

  
  // 
  // Waits until all of the vehicles tracked by the tracking array are in the given train.
  // Custom variation of TrainUtil.WaitUntilHas() that uses handbrake when decoupled.
  //
  public void WaitUntilHas(Vehicle theLoco, ShuntTrack[] tracking)
  {
    int i;
    bool pickupDone = false;
    Train train;
    Message msg;

    Interface.Log("Start of WaitUntilHas(), array is " + tracking.size() + " elements big!"); // test/debug

    // wait until all vehicles required are in a consist with the loco
    wait() 
    {
      on "Vehicle", "Coupled", msg:
      {
        pickupDone = true;
        train = theLoco.GetMyTrain();

        // verify that all are in the shunting consist
        for (i = 0; i < tracking.size(); i++)
        {
          if(train != tracking[i].vehicle.GetMyTrain())
          {
            pickupDone = false;
            break;
          }
        }

        if (pickupDone == true)
          break;

        continue;
      }
      on "Vehicle", "Decoupled", msg:
      {
        Sleep(0.5);
        for (i = 0; i < tracking.size(); i++)
          if (theLoco.GetMyTrain() != tracking[i].vehicle.GetMyTrain())
            tracking[i].vehicle.SetHandBrake(true);

        continue;
      }
    }

    Interface.Log("End of WaitUntilHas()"); // test/debug
  }


  //
  // Manage the pickup of the single box car and two coal cars from sidings at Robbins station
  //
  void RobbinsPickup(void)
  {
    bool carsPickedUp = false;
    int i;
    ShuntTrack[] tracking;
    Vehicle[] tempVehicleArr;

    Interface.Log("Start of RobbinsPickup()"); // test/debug


    //
    // Pickup two coal cars and boxcar
    //

    // Play conversation 2 as user approaches Robin in pass-over loop
    Navigate.OnTrigger(me, userTrain, "TrigConversation2", Navigate.TRIGGER_ENTER);
    World.Play2DSound("Conv2.wav");

    // open up coupling and decoupling so user can perform switching operations on the train
    userTrain.SetCouplingMask(Vehicle.COUPLE_BOTH); 
    userTrain.SetDecouplingMask(Vehicle.COUPLE_BOTH);

    // start score counting thread
    ScoringCounter();

    // loop until user's loco is at head of consist containing all required cars
    while(!carsPickedUp)
    {
      // wait until all cars required are in a consist with the loco
      WaitUntilHas(loco, assortedCars);

      // verify user consist has loco at head and only the cars needed (via length)
      tempVehicleArr = loco.GetMyTrain().GetVehicles();
      i = tempVehicleArr.size();

      if ((loco == loco.GetMyTrain().GetFrontmostLocomotive()) and (i == 15))
        carsPickedUp = true;
      else
        Interface.Print(stringTable.GetString("hvc_warning_locohead"));
    }


    // kill off score counting thread
    SendMessage(me, "Kill", "SwitchingScore");

    // update user train pointer
    userTrain = loco.GetMyTrain();

    // block off all coupling and decoupling operations once train is assembled
    userTrain.SetCouplingMask(Vehicle.COUPLE_NONE); 
    userTrain.SetDecouplingMask(Vehicle.COUPLE_NONE);

    // play conversation 3 slightly after the consist is assembled (don't sleep for conversation as
    // we want to process user movements as conversation is being played)
    Sleep(Math.Rand(4, 5));
    World.Play2DSound("Conv3.wav");

    // update user's train pointer
    userTrain = loco.GetMyTrain();

    Interface.Log("End of RobbinsPickup()"); // test/debug
  }


  //
  // Manages the passing at Stormont with AI passenger train
  //
  void StormontPassing(void)
  {
    Train stormontAiPass;  // AI train that passes user

    Interface.Log("Start of StormontPassing()"); // test/debug

    // warn user to slow down and be prepared
    Navigate.OnTrigger(me, userTrain, "SPEEDWARN", Navigate.TRIGGER_ENTER);
    objectiveIcon = "pause.tga";
    Interface.SetObjectiveIcon("pause.tga");  
  
    // warn user to stop
    Navigate.OnTrigger(me, userTrain, "TrigStormStop", Navigate.TRIGGER_ENTER);
    objectiveIcon = "stop.tga";
    Interface.SetObjectiveIcon("stop.tga");

    // create ai passenger train and run its schedule
    stormontAiPass = World1.CreateTrain("StormontPass", "Stormont Cross", false);
    stormontAiPass.RunSchedule(stormontAiPassSchedule, false, 0);

    // wait for passenger train to clear line and change junction so user can go on...
    Timetable.OnAnnouncement(me, "StormontPassClear");
    Navigate.LockJunction("Stormont North", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Odlum South", Junction.DIRECTION_RIGHT, true);
    Navigate.LockJunction("Odlum North", Junction.DIRECTION_RIGHT, true);
    objectiveIcon = "play.tga";
    Interface.SetObjectiveIcon("play.tga");

    // end checkpoint
    Navigate.OnTrigger(me, userTrain, "SPEEDEND", Navigate.TRIGGER_ENTER); // should be exit

    Interface.Log("End of StormontPassing()"); // test/debug
  }


  //
  // Handles the dropping off of 6 coal cars at a mine.
  //
  void MineDrop(ShuntTrack[] tracking, string dropZoneName1, string dropZoneName2, string dropZoneName3, string mineName, int remainSize)
  {
    bool coalCarsInPlace = false;
    bool trainAssembled = false;
    int i, dropCount1, dropCount2, dropCount3;
    Vehicle[] tempVehicleArray;
    Message msg;

    Trigger dropZone1 = cast<Trigger> Router.GetGameObject(dropZoneName1);
    Trigger dropZone2 = cast<Trigger> Router.GetGameObject(dropZoneName2);
    Trigger dropZone3 = cast<Trigger> Router.GetGameObject(dropZoneName3);

    Interface.Log("Start of MineDrop()"); // test/debug


    // loop until 6 coal hoppers are dropped off and train is assembled again with loco at front
    while (!trainAssembled)
    {
      // wait for a decouple/couple messages (we only check coal cars when we get such a message,
      // would be wasteful otherwise)
      wait() 
      {
        on "Vehicle", "Decoupled", msg :
        {
          // apply brakes to all vehicles not attached to the loco so they don't roll away
          Sleep(0.5);
          for (i = 0; i < assortedCars.size(); i++)
            if (loco.GetMyTrain() != tracking[i].vehicle.GetMyTrain())
              tracking[i].vehicle.SetHandBrake(true);

          // reset completion flag and car counters
          coalCarsInPlace = false;
          dropCount1 = 0;
          dropCount2 = 0;
          dropCount3 = 0;

          // iterate through all cars 
          // the rules of completion are basically that as longs there are 2 coal cars in each mine dropzone,
          // it doesn't matter (total of any 6 out of 12 which are in the coal
          for (i = 0; i < tracking.size(); i++)
          {
            // is current car a coal car?
            if (tracking[i].name == "Coal")
            {
              // check sidings for coal car's presence
              if (TrainUtil.IsInTrigger(me, tracking[i].vehicle, dropZone1))
                dropCount1++;
              else if (TrainUtil.IsInTrigger(me, tracking[i].vehicle, dropZone2))
                dropCount2++;
              else if (TrainUtil.IsInTrigger(me, tracking[i].vehicle, dropZone3))
                dropCount3++;
            }
          }

          // If we have 2 coal cars in each dropzone (rely on counters here) than all coal cars are
          // in place so then check if the remaining consist is assembled. If so, we cab break out 
          // of here.
          if ((dropCount1 == 2) and (dropCount2 == 2) and (dropCount3 == 2))
          {
            coalCarsInPlace = true;
            Interface.Print(stringTable.GetString("hvc_coalhopper_inplace"));
            Interface.Log(stringTable.GetString("hvc_coalhopper_inplace"));

            tempVehicleArray = loco.GetMyTrain().GetVehicles();
            userTrain = loco.GetMyTrain();

            // is user's train consist ready?
            if ((tempVehicleArray.size() == remainSize) and
                (loco == userTrain.GetFrontmostLocomotive())) // is loco at head of train?
            {
              trainAssembled = true;
              break; // break out of this wait cycle
            }
          }

          // Display progress update of coal cars in place
          if (dropCount1 > 2) 
            dropCount1 = 2;
          if (dropCount2 > 2) 
            dropCount2 = 2;
          if (dropCount3 > 2) 
            dropCount3 = 2;
          Interface.Print(stringTable.GetString2("hvc_coalhopper_not_done", (dropCount1 + dropCount2 + dropCount3), 6));
          Interface.Log(stringTable.GetString2("hvc_coalhopper_not_done", (dropCount1 + dropCount2 + dropCount3), 6));

          continue;
        }
        on "Vehicle", "Coupled", msg:
        {
          // verify all coal cars are in place, if not, don't bother checking the train any further
          if (coalCarsInPlace)
          {
            tempVehicleArray = loco.GetMyTrain().GetVehicles();
            userTrain = loco.GetMyTrain();

            // is user's train consist ready
            if ((tempVehicleArray.size() == remainSize) and
                (loco == userTrain.GetFrontmostLocomotive())) // redundant?
            {
              trainAssembled = true;
              break;
            }
          }
          continue;
        }
      }
    }


    // lock off and tag all cars that are not in consist (ie dropped off at the mine)
    for (i = 0; i < tracking.size(); i++)
    {
      // if the vehicle isn't in the train and is a coal hopper, label it at the mine
      if (!TrainUtil.IsInTrain(userTrain, tracking[i].vehicle) and (tracking[i].name == "Coal"))
      {
        Interface.Log(stringTable.GetString2("hvc_coalhopper_tag", i, mineName));
        tracking[i].name = "Coal" + mineName;
        tracking[i].vehicle.SetDecouplingMask(Vehicle.COUPLE_NONE);
        tracking[i].vehicle.SetCouplingMask(Vehicle.COUPLE_NONE);
      }
    }

    Interface.Log("End of MineDrop()"); // test/debug
  }


  //
  // First mine drop-off at Odlum
  //
  void FirstMineDrop(void)
  {
    int i;
    ShuntTrack[] tracking;

    Interface.Log("Start of FirstMineDrop()"); // test/debug


    // open up coupling and decoupling so user can perform switching operations on the train
    userTrain.SetCouplingMask(Vehicle.COUPLE_BOTH); 
    userTrain.SetDecouplingMask(Vehicle.COUPLE_BOTH);

    // start score counting thread
    ScoringCounter();

    // Manage car drop off in mine
    MineDrop(assortedCars, "TrigOdlumMine1", "TrigOdlumMine2", "TrigOdlumMine3", "Odlum", 9);

    // update user's train pointer
    userTrain = loco.GetMyTrain();

    // kill off score counting thread
    SendMessage(me, "Kill", "SwitchingScore");

    // block off all coupling and decoupling operations once train is assembled
    userTrain.SetCouplingMask(Vehicle.COUPLE_NONE); 
    userTrain.SetDecouplingMask(Vehicle.COUPLE_NONE);

    Interface.Log("End of FirstMineDrop()"); // test/debug
  }


  //
  // Second mine drop-off at Odlum
  //
  void SecondMineDrop(void)
  {
    int i;
    ShuntTrack[] tracking;

    Interface.Log("Start of SecondMineDrop()"); // test/debug


    // entry checkpoint
    Navigate.OnTrigger(me, userTrain, "TrigAftonExit", Navigate.TRIGGER_ENTER);

    // open up coupling and decoupling so user can perform switching operations on the train
    userTrain.SetCouplingMask(Vehicle.COUPLE_BOTH); 
    userTrain.SetDecouplingMask(Vehicle.COUPLE_BOTH);

    // start score counting thread
    ScoringCounter();

    // Manage car drop off in mine
    MineDrop(assortedCars, "TrigAftMineDrop1", "TrigAftMineDrop2", "TrigAftMineDrop3", "Afton", 3);

    // update user's train pointer
    userTrain = loco.GetMyTrain();

    // kill off score counting thread
    SendMessage(me, "Kill", "SwitchingScore");

    // block off all coupling and decoupling operations once train is assembled
    userTrain.SetCouplingMask(Vehicle.COUPLE_NONE); 
    userTrain.SetDecouplingMask(Vehicle.COUPLE_NONE);

    Interface.Log("End of SecondMineDrop()"); // test/debug
  }


  //
  // Various activities at Highland Valley
  //
  void HighlandValley(void)
  {
    bool hvDropOffDone = false;
    Message msg;
    Trigger dropZone = cast<Trigger> Router.GetGameObject("TrigHVBoxCars");

    Interface.Log("Start of HighlandValley()"); // test/debug


    // Create F7 at highland valley.
    pickupF7 = World1.CreateTrain("F72", "Thai Plant 2", true);

    // entry checkpoint
    Navigate.OnTrigger(me, userTrain, "TrigHVEntry", Navigate.TRIGGER_ENTER);

    // open up coupling and decoupling so user can perform switching operations on the train
    userTrain.SetCouplingMask(Vehicle.COUPLE_BOTH); 
    userTrain.SetDecouplingMask(Vehicle.COUPLE_BOTH);

    // start score counting thread
    ScoringCounter();

    // loop until box cars are dropped off
    while (!hvDropOffDone)
    {
      on "Vehicle", "Decoupled", msg :
      {
        // verify both box cars are in place
        if (TrainUtil.IsInTrigger(me, assortedCars[12].vehicle, dropZone) and
            TrainUtil.IsInTrigger(me, assortedCars[13].vehicle, dropZone))
        {
          Interface.Log("Box cars dropped off at Highland Valley ok."); // temp/debug

          // play conversation 4 slightly after user has dropped off box cars
          World.Play2DSound("Conv4.wav");
 
          // lock off
          assortedCars[12].vehicle.SetCouplingMask(Vehicle.COUPLE_NONE);
          assortedCars[12].vehicle.SetDecouplingMask(Vehicle.COUPLE_NONE);
          assortedCars[13].vehicle.SetCouplingMask(Vehicle.COUPLE_NONE);
          assortedCars[13].vehicle.SetDecouplingMask(Vehicle.COUPLE_NONE);

          hvDropOffDone = true;
        }
      }

    }

    // update user's train pointer
    userTrain = loco.GetMyTrain();

    // kill off score counting thread
    SendMessage(me, "Kill", "SwitchingScore");

    // block off all coupling and decoupling operations once train is assembled
    userTrain.SetCouplingMask(Vehicle.COUPLE_NONE); 
    userTrain.SetDecouplingMask(Vehicle.COUPLE_NONE);

    Interface.Log("End of HighlandValley()"); // test/debug
  }


  //
  // Pickup loaded coal cars at a mine.
  //
  void MinePickup(string mineName, int length)
  {
    int i;
    bool carsPickedUp = false;
    ShuntTrack[] tracking;
    Vehicle[] tempVehicleArray;

    Interface.Log("Start of MinePickup()"); // test/debug


    // create loaded coal cars in the mine
    Train loadedCoal1 = World1.CreateTrain("2LoadedCoalCars", mineName + "1", true);
    Train loadedCoal2 = World1.CreateTrain("2LoadedCoalCars", mineName + "2", true);
    Train loadedCoal3 = World1.CreateTrain("2LoadedCoalCars", mineName + "3", true);


    // initialize tracking array
    tracking = new ShuntTrack[6];
    for (i = 0; i < 6; i++)
    {
      tracking[i] = new ShuntTrack();
      tracking[i].name = null;
      tracking[i].trigger = null;
    }

    tempVehicleArray = loadedCoal1.GetVehicles();
    tracking[0].vehicle = tempVehicleArray[0];
    tracking[1].vehicle = tempVehicleArray[1];

    tempVehicleArray = loadedCoal2.GetVehicles();
    tracking[2].vehicle = tempVehicleArray[0];
    tracking[3].vehicle = tempVehicleArray[1];

    tempVehicleArray = loadedCoal3.GetVehicles();
    tracking[4].vehicle = tempVehicleArray[0];
    tracking[5].vehicle = tempVehicleArray[1];


    // open up coupling and decoupling so user can perform switching operations on the train
    userTrain.SetCouplingMask(Vehicle.COUPLE_BOTH); 
    userTrain.SetDecouplingMask(Vehicle.COUPLE_BOTH);

    // start score counting thread
    ScoringCounter();

    // wait until all coal cars are picked up and in a consist with the user's loco
    while(!carsPickedUp)
    {
      // wait until all cars required are in a consist with the loco
      WaitUntilHas(loco, tracking);

      // verify user consist has loco at head and only the cars needed (via length)
      tempVehicleArray = loco.GetMyTrain().GetVehicles();
      i = tempVehicleArray.size();

      if ((loco == loco.GetMyTrain().GetFrontmostLocomotive()) and (length + 6))
        carsPickedUp = true;
    }

    // kill off score counting thread
    SendMessage(me, "Kill", "SwitchingScore");

    // update user's train pointer
    userTrain = loco.GetMyTrain();

    // block off all coupling and decoupling operations once train is assembled
    userTrain.SetCouplingMask(Vehicle.COUPLE_NONE); 
    userTrain.SetDecouplingMask(Vehicle.COUPLE_NONE);

    Interface.Log("End of MinePickup()"); // test/debug
  }


  //
  // Handles the train through the coal dump, activating the particles.
  //
  void GreenwoodCoalDump(void)
  {
    float velocity;

    // schedule to guide train through coal dump (small, but we need AI to get muscle power to haul
    // cars up slope onto the dump)
    Schedule tempSchedule = new Schedule();
    tempSchedule.StopAtTrigger("StartDump", 0);


    // change junction to access coal heap
    Navigate.LockJunction("Comminco Mill", Junction.DIRECTION_RIGHT, false);

    // wait for user to get near coal heap entry and warn them
    Navigate.OnTrigger(me, userTrain, "COMINCO SIGNAL", Navigate.TRIGGER_ENTER);
    objectiveIcon = "pause.tga";
    Interface.SetObjectiveIcon("pause.tga");

    // start off conversation 5
    World.Play2DSound("Conv5.wav");


    // seize control of train and takeover
    Navigate.OnTrigger(me, userTrain, "TakeControl", Navigate.TRIGGER_ENTER);

    World.SetCameraFlags(World.CAMERA_LOCKED);
    World.SetCamera(loco, World.CAMERA_TRACKING);

    // get velocity setting as user triggers warning, so we can reverse train if needed (so AI 
    // schedule doesn't get confused and go wrong way)
    velocity = loco.GetVelocity();
    Interface.Log("Velocity at handover point is " + velocity); // debug

    // determine if train's throttle is reversed when approaching coal dump,
    if (velocity < 0.0)
      userTrain.Turnaround();

    // train on dump is run by schedule
    userTrain.SetHandBrake(false);
    userTrain.RunSchedule(tempSchedule, false, 0);

    // stop train and start off coal particles
    Navigate.OnTrigger(me, userTrain, "StartDump", Navigate.TRIGGER_ENTER);
    userTrain.SetHandBrake(true);

    // activate coal dump particles
    Interface.Log("Doing coal particles here!");
    SendMessage(coalParticle1, "pfx", "+0");
    SendMessage(coalParticle2, "pfx", "+0");

    Sleep(5.0);
  }


  //
  // main thread
  // brief main is executed automatically after Load() is called.
  //
  thread void main(void)
  {
    int i, total;

    // Start the monitor thread to monitor speeding, derailing etc.
    Monitor();


    //
    // Gameplay
    //

    // 1000 points to start out with
    Interface.AdjustScore(1000);

    // start with pause icon and camera aimed at loco
    objectiveIcon = "pause.tga";
    Interface.SetObjectiveIcon("pause.tga");

    // set camera to tracking view with characters visible near shed
    World.SetCameraFlags(World.CAMERA_LOCKED);
    World.SetCamera(loco, World.CAMERA_TRACKING);

    // Let user select DCC or Cabin Control for the scenario
    SelectCabinDCC(userTrain);

    // start off the AI train which passes through yard while user is switching
    aiInner.RunSchedule(initialPassSchedule, false, 0);

    // start thread which runs AI trains around mainline
    RunAITrains();


    //
    // Start out with pickup in the yard
    //
    StartGreenwood();

    // update score for completing Greenwood pickup
    UpdateScore("hvc_objective_greenwood_start", stringTable.GetString1("hvc_next_objective_proceed", "Robbins"));

    // checkpoint
    Navigate.OnTrigger(me, userTrain, "COMINCO SIGNAL", Navigate.TRIGGER_LEAVE);
    Interface.Log("Checkpoint - end of Greenwood Yard pickup."); // temp/debug


    //
    // Pickup coal cars and box car at Robbins
    //
    RobbinsPickup();

    // update score for completing Robbins pickup
    UpdateScore("hvc_objective_complete_robbins", stringTable.GetString1("hvc_next_objective_proceed", "Stormont"));

    // checkpoint
    Navigate.OnTrigger(me, userTrain, "CABOOSE", Navigate.TRIGGER_LEAVE);


    //
    // Pass AI train at Stormont
    //
    StormontPassing();

    // 200 points for completing Stormont passing
    Interface.AdjustScore(200);
    Interface.Print(stringTable.GetString1("hvc_objective_stormont_passing", "200"));
    Interface.Log(stringTable.GetString1("hvc_objective_stormont_passing", "200"));

    Sleep(5);
    Interface.Print(stringTable.GetString1("hvc_next_objective_mine_drop", "Odlum"));  

    // checkpoint - kill off AI passenger train
    Navigate.OnTrigger(me, userTrain, "TrigKillStormontPass", Navigate.TRIGGER_ENTER);
    Timetable.Announce("KillStormontPass", me.GetId());


    //
    // First mine drop off at Odlum
    //
    FirstMineDrop();

    // update score for completing Odlum Mine drop-off
    UpdateScore("hvc_objective_odlum_dropoff", stringTable.GetString1("hvc_next_objective_mine_drop", "Afton"));

    // checkpoint
    Navigate.OnTrigger(me, userTrain, "TrigPastOdlum", Navigate.TRIGGER_LEAVE);


    //
    // Second mine drop off at Afton
    //
    SecondMineDrop();

    // update score for completing Afton Mine drop-off
    UpdateScore("hvc_objective_afton_dropoff", stringTable.GetString1("hvc_next_objective_proceed", "Highland Valley"));

    // checkpoint
    Navigate.OnTrigger(me, userTrain, "TrigAftonExit", Navigate.TRIGGER_LEAVE);


    //
    // Drop off box cars at Highland Valley
    //
    HighlandValley();

    // update score for completing Afton Mine drop-off
    UpdateScore("hvc_objective_boxcars_dropped", stringTable.GetString1("hvc_next_objective_proceed", "Highland Valley"));

    // Kill off all empty coal cars at the mines
    for (i = 0; i < 12; i++)
      World.DeleteTrain(assortedCars[i].vehicle.GetMyTrain());

    // exit checkpoint
    Navigate.OnTrigger(me, userTrain, "TrigHVEntry", Navigate.TRIGGER_LEAVE);


    //
    // Pickup loaded coal cars from Afton Mine (on way back to Greenwood)
    //
    MinePickup("StartAftMine", 7);

    // update score for completing Afton Mine pickup
    UpdateScore("hvc_objective_afton_pickup", stringTable.GetString1("hvc_next_objective_mine_pickup", "Odlum"));

    // checkpoint
    Navigate.OnTrigger(me, userTrain, "TrigAftBridge", Navigate.TRIGGER_LEAVE);

    // unlock junctions
    Navigate.LockJunction("Odlum South", Junction.DIRECTION_LEFT, true);
    Navigate.LockJunction("Odlum North", Junction.DIRECTION_RIGHT, true);


    //
    // Pickup loaded coal cars from Odlum Mine (on way back to Greenwood)
    //
    MinePickup("StartOdlumMine", 13);

    // update score for completing Odlum Mine pickup
    UpdateScore("hvc_objective_odlum_pickup", stringTable.GetString("hvc_next_objective_coal_dump"));


    //
    // Greenwood coal dump
    //
    GreenwoodCoalDump();

    // 300 points for arriving at the coal dump
    Interface.AdjustScore(300);
    Interface.Print(stringTable.GetString1("hvc_objective_coaldump", "300"));
    Interface.Log(stringTable.GetString1("hvc_objective_coaldump", "300"));
    Sleep(4.0);


    // display scorecard with statistics on user's activities and scoring/penalties etc.
    DoScoreCard();

    // finish off the scenario
    Interface.SetObjective(stringTable.GetString1("scenario_complete_objective", Interface.GetScore()), "tick.tga");
    Interface.SetResults();
    Interface.LogResult(stringTable.GetString1("scenario_complete", Interface.GetScore()));
    Interface.Print(stringTable.GetString1("scenario_complete", Interface.GetScore()));
    scenarioDone = true;
    World.EndScenario(8.0);
  }

};

