//
// Queensland.gs
//
//  Copyright (C) 2002 Auran Developments Pty Ltd
//  All Rights Reserved.
//

include "trainz.gs"
include "world1.gs"
include "common.gs"
include "timetablescheduler.gs"
include "browser.gs"
include "stringtable.gs"


//
// Custom commands for AI schedules
//

// Sends message to train to blast it's horn
class AiHornMsg isclass CustomCommand
{
  bool Execute(Train train, int px, int py, int pz)
  {
    train.Sleep(Math.Rand(px, py));
    train.SendMessage(train, "loco", "horn");
    return true;
  }
};


//
// class MyQueensland
// brief This is the scenario class.
//
game class MyQueensland isclass Scenario, ScenarioSupport
{
  bool scenarioDone = false;
  bool driverWarning = false;  // is the driver currently under warning? If so, then don't interrupt them.
                               // This preevents repeated continous warnings so they are spaced out a bit.
  string lastspeedImage;       // icon for most recent speed zone (digital display .tga file)
  int zoneCount = 1;           // counter to keep track of which speed zone user is going through (used for logging)

  Train userTrain, longPass, longMixedFreight, smallPass, invisibleTrain, platformPerson;
  Schedule aiPassStartSchedule, longPassSchedule, longMixedFreightSchedule;
  Signal signalTN, signalTS1, signalTS2, signalSB1;
  StationBehaviour stationBehaviour;

  StringTable stringTable;  // scenario string table

  // convinient conversion constants
  public define float MPS_TO_KPH   = 3.6;                        // metres/second -> kilometres/hour
  public define float WARNING_ZONE = (10.0 * Train.KPH_TO_MPS);  // warning zone in MPS (> speed limit + 10kph)
  public define float BRAKE_ZONE   = (6.0 * Train.KPH_TO_MPS);   // emergency brake zone in MPS (> speed limit + 16kph)

  // method definition
  thread void EmergencyBrakes(Train train);


  //
  // Load will be called by Trainz to load the scenario map, and when the user presses Ctrl-L
  // param data is the save game data if loading a saved game.
  //
  bool Load(string data)
  {
    if(data and data.size())
    {
      Interface.Load(data);
    }

    // initialize scenario string table
    stringTable = ScenarioStringTable.Get();

    // load the map
    if(!World.LoadMap(World.FindKUID("queensland")))
    {
      Interface.Log(stringTable.GetString1("std_error_loading_map_name", "queensland"));
      return false;
    }

    // disable saving on this scenario
    BrowserInterface.SetSaveAllowed(false);


    //
    // create consists
    //

    // user controlled train that is monitored
    userTrain = World1.CreateTrain("longpass1", "START", true);
//    userTrain = World1.CreateTrain("longpass1", "StartTemp2", true);  // test/debug
//    userTrain = World1.CreateTrain("longpass1", "StartTemp6", true);  // test/debug
//    userTrain = World1.CreateTrain("longpass1", "StartFour", true);  // test/debug
//    userTrain = World1.CreateTrain("longpass1", "StartFive", true);  // test/debug
    userTrain.SetFloatingLimit(WARNING_ZONE, true); // set floating speed limit
    EmergencyBrakes(userTrain);                     // start speed monitoring thread

    // long passenger train that passes user at start
    longPass = World1.CreateTrain("longpass2", "Start LongPass", true);

    // long freight train that passes user at Warnertown
    longMixedFreight = World1.CreateTrain("longmixedfreight", "Start BeforeTunnel", true);

    // small passenger train that is static consist of interest
    smallPass = World1.CreateTrain("shortpass", "Start Blackall", true);

    // special miscellaneous "trains" (invisible train & driver character train)
    invisibleTrain = World1.CreateTrain("invisible", "StartInvisible", true); 
    invisibleTrain.SetAutopilotMode(Train.CONTROL_SCRIPT);
    platformPerson = World1.CreateTrain("platformperson", "StartPersonPlatform", true);
    platformPerson.SetAutopilotMode(Train.CONTROL_SCRIPT);


    //
    // Schedules
    //

    // initial AI control of train into the station
    aiPassStartSchedule = new Schedule();

    aiPassStartSchedule.OnTrigger("ONE", true);
    aiPassStartSchedule.Announce("TrainEnteredStation");
    aiPassStartSchedule.StopAtTrigger("Trig StopBremer", 0);
    aiPassStartSchedule.Announce("HandOver");
    
    // long passenger train that goes past
    longPassSchedule = new Schedule();

    longPassSchedule.StopAtTrigger("Trig StopLongPass", 0);
    longPassSchedule.DeleteTrain();

    // long freight that goes past at junction near tunnel entrance (Warnertown station)
    longMixedFreightSchedule = new Schedule();
    AiHornMsg aiHorn = new AiHornMsg();

    longMixedFreightSchedule.TakeJunction("TunnelSouth", Junction.DIRECTION_LEFT);
    longMixedFreightSchedule.OnTrigger("TrigFreightHorn1", true);
    longMixedFreightSchedule.Custom(aiHorn, 0, 2, 0);
    longMixedFreightSchedule.OnTrigger("TrigFreightHorn2", true);
    longMixedFreightSchedule.Custom(aiHorn, 0, 1, 0);
    longMixedFreightSchedule.OnTrigger("Trig Clear", true);
    longMixedFreightSchedule.Announce("LongFreightClear");
    longMixedFreightSchedule.StopAtTrigger("Trig StopLongFreight", 0);  // DeleteTrain();


    //
    // Create station behaviour for the user controlled passenger train at Warnertown
    //
    stationBehaviour = new StationBehaviour();

    stationBehaviour.SetDefaults();
    stationBehaviour.warning.objective = "";
    stationBehaviour.warning.icon = "pause.tga";
    stationBehaviour.stopped.objective = "";
    stationBehaviour.stopped.icon = "stop.tga";
    stationBehaviour.stopped.wav = "bell.wav";
    stationBehaviour.stopped.score = 0;        // this is handled in scenario code below by the timetable scheduler
    stationBehaviour.done.objective = "";
    stationBehaviour.done.icon = "pause.tga";
    stationBehaviour.done.score = 0;           // this is handled in scenario code below by the timetable scheduler
    stationBehaviour.overrun.icon = "bad.tga";
    stationBehaviour.overrun.score = -300;     // severe penalty, lose 300 points!
    stationBehaviour.reverse.icon = "bad.tga";
    stationBehaviour.mwstopped.icon = "bad.tga";
    stationBehaviour.mwstopped.alert = stringTable.GetString("std_bad_guard_clearence");
    stationBehaviour.mwstopped.colour = Interface.Colour_Red;
    stationBehaviour.mwstopped.score = -400;   // very severe penalty, lose 400 points!


    //
    // Signals
    //
    signalTN = cast<Signal> Router.GetGameObject("TN");
    signalTS1 = cast<Signal> Router.GetGameObject("TS1");
    signalTS2 = cast<Signal> Router.GetGameObject("TS2");
    signalSB1 = cast<Signal> Router.GetGameObject("SB1");

    signalSB1.SetSignalState(Signal.GREEN, "Clear through to Paisley.");

    // setup signals so if user passes one, scenario is terminated!
    AddHandler(userTrain, "Train", "Entered red signal", "PassSignal");


    //
    // Junctions
    //

    // Lock & prepare junctions for user route
    Navigate.LockJunction("Blackall North", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Blackall South", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("TunnelSouth", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("PaisleyNorth", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("PaisleySouth", Junction.DIRECTION_LEFT, false);  // AI train won't pass this

    return true;
  }


  //
  // Save will be called by Trainz when the user presses Ctrl-S.
  // return the save game string, such that load will be able to restor the save game
  // from the last save check point.
  //  
  string Save()
  {
    return Interface.Save();
  }


  //
  // TrainDerailed() will be called by Trainz when a train derails
  //
  void TrainDerailed(int trainId) 
  {
    if(!scenarioDone)
    {
      Interface.SetObjective(stringTable.GetString("std_terminated_derailed_train"), "bad.tga");
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);
      scenarioDone = true;
    }
  }


  //
  // TrainCollided() will be called by Trainz when a train collides
  //
  void TrainCollided(int trainId) 
  {
    if(!scenarioDone)
    {
      Interface.SetObjective(stringTable.GetString("std_terminated_train_collision"), "bad.tga");       
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);
      scenarioDone = true;
    }
  }


  //
  // Error message and termination method for when user crosses a trigger that they shouldn't have!
  //
  void OffLimits(Message msg)
  {
    // verify it was the user's passenger train
    if (msg.src == userTrain)
    {
      Interface.SetObjective(stringTable.GetString("std_bad_backwards"), "bad.tga");
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);
    }
  }


  //
  // TrainSpeedingFine() is called by Trainz every second your trains speed exceeds the floating limit
  //
  void TrainSpeedingFine() 
  {
    Interface.AdjustScore(-1);  
  }


  //
  // Thread that monitors train speed and responds to Monitor::Speeding messages generated
  //
  thread void EmergencyBrakes(Train train)
  {
    float current_speed, current_limit, emergency_brakes;
    Message msg;
    Vehicle loco = train.GetFrontmostLocomotive();

    // wait for Monitor::Speeding, Tick messages to be received
    wait()
    {
      on "Monitor::Speeding", "Tick", msg :
      {
        // is driver already being warned - don't want to interrupt the warning with another warning!
        if (!driverWarning)
        {
          // set the warning flag to true while driver is being warned/penalised
          driverWarning = true;

          // set up values so we can do a special check
          current_speed = loco.GetVelocity();
          current_limit = train.GetSpeedLimit();
          emergency_brakes = current_limit + WARNING_ZONE + BRAKE_ZONE;

          // convert to kph so data is more presentable debug log
          current_speed = current_speed * MPS_TO_KPH;
          emergency_brakes = emergency_brakes * MPS_TO_KPH;
          current_limit = current_limit * MPS_TO_KPH;

          Interface.Log("Train speed is " + current_speed + "kph and current speed limit is " + current_limit + "kph");

          // train is more than 10kph over speed limit, check if alarm or emergency brakes are required
          if (current_speed < emergency_brakes)
          {
            // The train is more than 10kph over speed limit, sound alarm and give warning
            World.Play2DSound("bell.wav");
            Interface.SetObjective(stringTable.GetString("qld_speed_10kph_over"), "bad.tga");

            // 50 points for the warning
            Interface.AdjustScore(-50);
            Interface.Print(stringTable.GetString2("qld_speed_points", "10", "50"));
            Interface.Log(stringTable.GetString2("qld_speed_points", "10", "50"));
          }
          else 
          {
            // must be 16ph or over the speed limit, give warning
            Interface.SetObjective(stringTable.GetString("qld_speed_16kph_over"), "bad.tga");
            Interface.AdjustScore(-200);  
            Interface.Print(stringTable.GetString2("qld_speed_points", "16", "200"));
            Interface.Log(stringTable.GetString2("qld_speed_points", "16", "200"));

            // seize control and stop the train
            train.SetAutopilotMode(Train.CONTROL_SCRIPT);
            train.SetDCCThrottle(0.0f);
            train.SetHandBrake(true);

            // wait until train has stopped before handing control back to user
            Sniff(train, "Train", "", true);
            wait() 
            {
              on "Train", "StoppedMoving", msg:
              {
                Interface.SetObjective(stringTable.GetString("qld_speed_stopped"), lastspeedImage);
                train.SetAutopilotMode(Train.CONTROL_MANUAL);
                train.SetHandBrake(false);
                break;
              }
            }
            Sniff(train, "Train", "", false);
          }

          // sleep for a while and re-enable the flag so the message can be processed again (if needed)
          Sleep(3.0);
          driverWarning = false;

          // hide warning text and restore speed zone icon display
          Sleep(3.0);
          Interface.SetObjective("", lastspeedImage);
          Interface.ShowObjective(false);
        }
        continue;
      }
    }

  }


  //
  // TrainBadCouple() is called by Trainz when vehicles couple greater than 8KPH.
  // Note: shouldn't be a problem in this scenario, no coupling is possible.
  //
  void TrainBadCouple(int vehicleId)
  {
    Interface.SetObjective(stringTable.GetString("std_bad_couple"), "bad.tga");
    World.Play2DSound("ALARM.wav");
    Interface.AdjustScore(-50);
  }


  //
  // Called when train passes a red signal
  //
  void PassSignal(Message msg)
  {
    Interface.SetObjective(stringTable.GetString("std_bad_redsignal"), "bad.tga");
    World.Play2DSound("ALARM.wav");
    World.EndScenario(5);
  }


  //
  // Manages stopping of train at Paisley
  //
  void ManagePaisleyStop(Train train)
  {
    bool paisleyPlatform = false;
    Message msg;
    Trigger platformArea = cast<Trigger> Router.GetGameObject("TrigPlatform");
    Trigger stationEntry = cast<Trigger> Router.GetGameObject("TrigMoveEnd");
    Trigger stationEnd   = cast<Trigger> Router.GetGameObject("TrigMoveFront");


    // wait for user to enter the initial warning trigger
    Navigate.OnTrigger(me, train, "TrigWarnPais", Navigate.TRIGGER_ENTER);
    Interface.SetObjectiveIcon("pause.tga");

    // wait for user to enter the trigger zone that defines the platform area
    Navigate.OnTrigger(me, train, "TrigMoveEnd", Navigate.TRIGGER_ENTER);
    Interface.SetObjectiveIcon("stop.tga");
 
    // loop until train has stopped at paisley
    while (!paisleyPlatform)
    {
      Sniff(train, "Train", "", true);
      wait()
      {
        on "Train", "StoppedMoving", msg :
        {
          // Is any part of the train in the platform area?
          if (TrainUtil.IsPartTrainInTrigger(me, train, platformArea))
          {
            // if train is clear of both end triggers, we can break out of here
            if (!TrainUtil.IsPartTrainInTrigger(me, train, stationEntry) and 
                !TrainUtil.IsPartTrainInTrigger(me, train, stationEnd))
            {
              break;
            }
            // if train is clear of entry trigger, but overlaps end trigger, it has gone too far
            else if (!TrainUtil.IsPartTrainInTrigger(me, train, stationEntry) and 
                     TrainUtil.IsPartTrainInTrigger(me, train, stationEnd))
            {
              // 50 points lost for not getting it right!
              Interface.AdjustScore(-50);
              Interface.Print(stringTable.GetString1("qld_missed_platform", "50"));
              Interface.Log(stringTable.GetString1("qld_missed_platform", "50"));
              Sleep(World.Play2DSound("guard.wav"));
            }
            // if train is clear of end trigger but overlaps entry trigger, it hasn't gone far enough
            else if (!TrainUtil.IsPartTrainInTrigger(me, train, stationEnd) and
                     TrainUtil.IsPartTrainInTrigger(me, train, stationEntry))
            {
              // 50 points lost for not getting it right!
              Interface.AdjustScore(-50);
              Interface.Print(stringTable.GetString1("qld_missed_platform", "50"));
              Interface.Log(stringTable.GetString1("qld_missed_platform", "50"));
              Sleep(World.Play2DSound("guard.wav"));
            }
          }
          continue;
        }
      }
      Sniff(train, "Trigger", "", false);

      paisleyPlatform = true;
    }

    // user has arrived at Paisley and stopped train so carriages are lined up with platforms - 300 points
    Interface.AdjustScore(300);
    Interface.Print(stringTable.GetString2("qld_arrived_platform", "Paisley", "300"));
    Interface.Log(stringTable.GetString2("qld_arrived_platform", "Paisley", "300"));

    // timetable scoring
    TimetableScheduler.WaitUntilTerminate(10, 09, 00, me, true, 200);
  }


  //
  // Groups together behaviour that is used for each speed zone into one method.
  //  image        - name of .tga speed icon
  //  currentTrig  - current trigger that defines start of the speed zone
  //  previousTrig - trigger for previous speed zone
  //  slowdown     - indicates if this speed zone is faster or slower than the previous one
  //
  void SpeedZone(string image, string currentTrig, string previousTrig, bool slowdown)
  {
    int trig = Navigate.TRIGGER_LEAVE; // defines whether train is to enter/leave zone trigger


    // determines whether to wait for train to enter/leave the speed zone trigger to activate alarm/icon
    if (slowdown)
      trig = Navigate.TRIGGER_ENTER;

    // wait for train to enter/leave the trigger at the start of the new speed zone
    Navigate.OnTrigger(me, userTrain, currentTrig, trig); 

    // play alarm buzzer and update speed icon
    World.Play2DSound("ALARM.wav");
    Interface.ShowObjective(false);
    lastspeedImage = image;
    Interface.SetObjectiveIcon(lastspeedImage);

    // adjust score and provide scoring info to user (100 points for each zone reached)
    Interface.AdjustScore(100);
    Interface.Print(stringTable.GetString2("qld_speed_zone", zoneCount, "100"));
    Interface.Log(stringTable.GetString2("qld_speed_zone", zoneCount, "100"));
    zoneCount++; // increment speed zone counter

    // ensure user can't go back down the track
    AddHandler(Router.GetGameObject(previousTrig), "Object", "Enter", "OffLimits");
  }


  //
  // main thread
  // brief main is executed automatically after Load() is called.
  //
  thread void main(void)
  {
    // Start the monitor thread to monitor speeding, derailing etc.
    Monitor();

    // Start off with 1000 points.  This will give us some points to play with before
    // the user obtains a negative score (if they do things wrong)
    Interface.AdjustScore(1000);

    // start with pause icon
    Interface.SetObjectiveIcon("pause.tga");

    // set camera to invisible train and display pause icon
    World.SetCameraFlags(World.CAMERA_LOCKED);
    World.SetCamera(invisibleTrain, World.CAMERA_TRACKING);

    // Let user select DCC or Cabin Control for the scenario
    SelectCabinDCC(userTrain);


    //
    // Intro sequence - train rolls into station and stops for you to take over
    //

    // start train to come in under AI control
    userTrain.RunSchedule(aiPassStartSchedule, false, 0);

    // driver on platform whistling
    Sleep(Math.Rand(4, 8));
    World.PlaySound("whistling.wav", 1000.0, 1.0, 1000.0, null, null);

    // wait for passenger train to enter station and switch cameras to it 
    Timetable.OnAnnouncement(me, "TrainEnteredStation");
    World.SetCamera(userTrain, World.CAMERA_TRACKING);

    // wait for train to stop so control can be handed over to user
    Timetable.OnAnnouncement(me, "HandOver");
    Sleep(Math.Rand(2, 5));

    // give camera control back to user (place them in cab though)
    World.SetCameraFlags(World.CAMERA_NORMAL);
    World.SetCamera(userTrain, World.CAMERA_INTERNAL);

    // delete driver person on station platform as he is no longer needed (you are supposed to think that it is you!)
    World.DeleteTrain(platformPerson);

    // have a chat and wait for a few radom seconds
    Sleep(World.PlaySound("gdmorning.wav", 1000.0, 1.0, 1000.0, null, null));
    Sleep(Math.Rand(2, 5));

    // display clipabord of scenario indstructions, start thread to monitor K keystroke
    DoInstructions();
    ManageBrowser();

    // hand over control of train to them
    userTrain.SetAutopilotMode(Train.CONTROL_MANUAL);
    Interface.SetObjectiveIcon("play.tga");


    //
    // Bremer to Warnertown
    //

    // 2nd speed zone trigger - 80kph
    SpeedZone("ts_080.tga", "TWO_80", "ONE", false);

    // 3rd speed zone trigger - 70kph
    SpeedZone("ts_070.tga", "THREE_70", "TWO_80", true);

    // trigger off a passenger train to pass user somewhere bettween Warnertown and Warranga for interest
    longPass.RunSchedule(longPassSchedule, false, 0);


    // 4th speed zone trigger - 60kph
    SpeedZone("ts_060.tga", "FOUR_60", "THREE_70", true);

    // 5th speed zone trigger - 90kph
    SpeedZone("ts_090.tga", "FIVE_90", "FOUR_60", false);

    // 6th speed zone trigger - 80kph
    SpeedZone("ts_080.tga", "SIX_80", "FIVE_90", true);

    // 7th speed zone trigger - 70kph (redundant because of station stop)
//    SpeedZone("ts_070.tga", "SEVEN_70", "SIX_80", false);


    //
    // Warnertown stop and freight passing
    //

    // Slow down and Stop at Warnertown
    Navigate.Station(me, userTrain, "Warnertown", "TrigWarnWarner", "TrigStopWarner", "TrigOverWarner", null, stationBehaviour, 15, 20);
    TimetableScheduler.WaitUntilTime(9, 47, 30, me, true, 200);
    World.Play2DSound("all_aboard.wav");
    Sleep(Math.Rand(4, 8));
    World.Play2DSound("bell.wav");
    Interface.SetObjectiveIcon("play.tga");
    AddHandler(Router.GetGameObject("SIX_80"), "Object", "Enter", "OffLimits"); // ensure user can't go back down the track

    // setup and execute freight passing
    signalTN.SetSignalState(Signal.GREEN, "Clear through the tunnel.");
    signalTS1.SetSignalState(Signal.YELLOW, "Clear through the tunnel.");
    longMixedFreight.RunSchedule(longMixedFreightSchedule, false, 0);

    // wait until freight is clear and set signals/junctions appropriately
    Timetable.OnAnnouncement(me, "LongFreightClear");
    signalTN.SetSignalState(Signal.RED, "Line not clear.");
    signalTS1.SetSignalState(Signal.RED, "Line not clear.");
    signalTS2.SetSignalState(Signal.GREEN, "Line not clear.");
    Navigate.LockJunction("TunnelSouth", Junction.DIRECTION_RIGHT, false);
    Interface.SetObjectiveIcon("play.tga");

    // update user score (200 points)
    Interface.AdjustScore(200);
    Interface.Print(stringTable.GetString2("qld_passing_complete", "Warnertown", "200"));
    Interface.Log(stringTable.GetString2("qld_passing_complete", "Warnertown", "200"));



    //
    // Warnertown to Paisley
    //

    // 8th speed zone trigger - 80kph
    SpeedZone("ts_080.tga", "EIGHT_80", "SEVEN_70", false);

    // 9th speed zone trigger - 60kph
    SpeedZone("ts_060.tga", "NINE_60", "EIGHT_80", true);

    // 10th speed zone trigger - 70kph
    SpeedZone("ts_070.tga", "TEN_70", "NINE_60", false);

    // 11th speed zone tirgger - 80kph
    SpeedZone("ts_080.tga", "ELEVEN_80", "TEN_70", false);

    // 12th speed zone trigger - 70kph
    SpeedZone("ts_070.tga", "TWELVE_70", "ELEVEN_80", true);


    // 13th speed zone trigger - 60kph (Blackall South junction)
    SpeedZone("ts_060.tga", "THIRTEEN_60", "TWELVE_70", true);

    // when passing through Blackall, waiting passenger train blasts its horn
    Navigate.OnTrigger(me, userTrain, "TrigSmallPassHorn", Navigate.TRIGGER_ENTER);
    Sleep(Math.Rand(0, 3));
    SendMessage(smallPass, "loco", "horn");


    // 14th speed zone trigger - 70kph (Blackall North junction)
    SpeedZone("ts_070.tga", "FOURTEEN_70", "THIRTEEN_60", false);

    // 15th speed zone trigger - 60kph
    SpeedZone("ts_060.tga", "FIFTEEN_60", "FOURTEEN_70", true);

    // 16th speed zone trigger - 50kph
    SpeedZone("ts_050.tga", "SIXTEEN_50", "FIFTEEN_60", true);


    // Slow down and stop at Paisley
    ManagePaisleyStop(userTrain);


    // finish off the scenario
    Interface.SetObjective(stringTable.GetString1("scenario_complete_objective", Interface.GetScore()), "tick.tga");
    Interface.SetResults();
    Interface.LogResult(stringTable.GetString1("scenario_complete", Interface.GetScore()));
    Interface.Print(stringTable.GetString1("scenario_complete", Interface.GetScore()));
    scenarioDone = true;
    World.EndScenario(8.0);
  }
};
