//
// RobeRiverIron.gs
//
//  Copyright (C) 2002 Auran Developments Pty Ltd
//  All Rights Reserved.
//

include "trainz.gs"
include "world1.gs"
include "common.gs"
include "browser.gs"
include "stringtable.gs"


//
// Custom commands for AI train schedules
//

// Conversation 1 (loaded AI train approaches user near start)
class Conversation1 isclass CustomCommand
{
  bool Execute(Train train, int px, int py, int pz)
  {
    train.SetAutopilotMode(Train.CONTROL_AUTOPILOT);
    train.Sleep(Math.Rand(px, py) + World.Play2DSound("conversation1.wav"));
    return true;
  }
};

// Sends message to train to blast its horn
class AiHornMsg isclass CustomCommand
{
  bool Execute(Train train, int px, int py, int pz)
  {
    train.Sleep(Math.Rand(px, py));
    train.SendMessage(train, "loco", "horn");
    return true;
  }
};


//
// class MyRobeRiver
// brief This is the scenario class.
//
game class MyRobeRiver isclass Scenario, ScenarioSupport
{
  bool scenarioDone = false;
  bool carefulMode = false;
  bool recentBadCouple = false;
  bool warningUser = false;
  bool talkingToDriver = false;
  string objectiveIcon = null;

  Train userOreTrain, userOreTrain2, aiOreTrain, trackGangCrew;
  Vehicle[] userOreCars;
  Schedule aiLoadedTrainSchedule;
  AiHornMsg aiHorn;
  Conversation1 conversation1;
  GameObject mineParticle; // mine scenery object used to access particle effects

  StringTable stringTable; // scenario string table

  // metres/second -> kilometers/hour conversion
  public define float MPS_TO_KPH = 3.6; 


  //
  // Load will be called by Trainz to load the scenario map, and when the user presses Ctrl-L
  // param data is the save game data if loading a saved game.
  //
  bool Load(string data)
  {
    if(data and data.size())
    {
      Interface.Load(data);
    }

    // initialize scenario string table
    stringTable = ScenarioStringTable.Get();

    // load the map
    if(!World.LoadMap(World.FindKUID("robescenario")))
    {
      Interface.Log(stringTable.GetString1("std_error_loading_map_name", "robescenario"));
      return false;
    }

    // try and gain pointer to the mesa mine (so particle effects can be activated later)
    mineParticle = Router.GetGameObject("MesaMine");
    if (!mineParticle)
    {
      Interface.Log(stringTable.GetString1("std_error_loading_particleFX", "Mine Ore Dust"));
      return false;
    }

    // disable saving on this scenario
    BrowserInterface.SetSaveAllowed(false);


    //
    // Load the train consists
    //

    // User controlled ore train
    userOreTrain = World1.CreateTrain("robeoreemptyuser", "StartUserEmpty", true);
//    userOreTrain = World1.CreateTrain("robeoreemptyuser", "START HERE", false); // test/debug
//    userOreTrain = World1.CreateTrain("robeoreemptyuser", "StartTemp2", true); // test/debug
//    userOreTrain = World1.CreateTrain("robeoreemptyuser", "StartTemp3", false); // test/debug
//    userOreTrain = World1.CreateTrain("robeoreemptyuser", "StartLunchOver", true); // test/debug
    userOreTrain.SetFloatingLimit(10.0 * Train.KPH_TO_MPS, true);

    // Loaded ore train that passes user at start
    aiOreTrain = World1.CreateTrain("robeoreloadedai", "StartAILoaded", true);
//    aiOreTrain = World1.CreateTrain("robeoreloadedai", "StartTemp", true); // test/debug

    // Idle track gang crew's train
    trackGangCrew = World1.CreateTrain("trackgang", "StartTrackGang", true);


    //
    // Schedules
    //

    // Schedule for AI train that passes user at start
    aiLoadedTrainSchedule = new Schedule();
    aiHorn = new AiHornMsg();
    conversation1 = new Conversation1();

    aiLoadedTrainSchedule.OnTrigger("TRIGCONVERSATION1", true);
    aiLoadedTrainSchedule.Custom(aiHorn, 2, 3, 0);
    aiLoadedTrainSchedule.Custom(aiHorn, 0, 4, 0);
    aiLoadedTrainSchedule.Custom(conversation1, 5, 10, 0);
    aiLoadedTrainSchedule.OnTrigger("TrigLoopSouthClear", true);
    aiLoadedTrainSchedule.Custom(aiHorn, 1, 3, 0);
    aiLoadedTrainSchedule.OnTrigger("TrigStartHorn2", true);
    aiLoadedTrainSchedule.Custom(aiHorn, 3, 6, 0);
    aiLoadedTrainSchedule.OnTrigger("TrigLoopSouthClear", false);
    aiLoadedTrainSchedule.Announce("LoadedClearJunction");
    aiLoadedTrainSchedule.OnTrigger("LoadedClear", true);
    aiLoadedTrainSchedule.DeleteTrain();


    //
    // Junctions
    //

    // pasing loop
    Navigate.LockJunction("LOOP South", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("LOOP North", Junction.DIRECTION_RIGHT, false);

    // mesa junctions (yard and mine)
    Navigate.LockJunction("MESA J Yard Nth", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Junction 830", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Junction 827", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Junction 826", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("MESA J 1", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("MESA J 2", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("MESA J 3", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("MESA J 4", Junction.DIRECTION_RIGHT, false);


    return true;
  }


  //
  // Save will be called by Trainz when the user presses Ctrl-S.
  // return the save game string, such that load will be able to restore the save game
  // from the last save check point.
  //  
  string Save()
  {
    return Interface.Save();
  }


  //
  // TrainDerailed() will be called by Trainz when a train derails
  //
  void TrainDerailed(int trainId)
  {
    if(!scenarioDone)
    {
      Interface.SetObjective(stringTable.GetString("std_terminated_derailed_train"), "bad.tga");
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);
      scenarioDone = true;
    }
  }


  //
  // TrainCollided() will be called by Trainz when a train collides
  //
  void TrainCollided(int trainId) 
  {
    if(!scenarioDone)
    {
      Interface.SetObjective(stringTable.GetString("std_terminated_train_collision"), "bad.tga");       
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);
      scenarioDone = true;
    }
  }


  //
  // TrainSpeedingFine() is called by Trainz every second your trains speed exceeds the floating limit
  //
  void TrainSpeedingFine() 
  {
    // make sure we are not in track-gang careful mode (another thread handles speeding in that case)
    if (!carefulMode)
      Interface.AdjustScore(-1);
  }


  //
  // Manage passing of the train through the caurtion zone where track maintenance is in progress
  //
  thread void CautionZone(Train train)
  {
    Message msg; 
    Vehicle loco = train.GetFrontmostLocomotive();
    float current_speed;

    // wait until kill message has been received, but in the mean time, keep monitoring speeding
    wait()
    {
      on "Monitor::Speeding", "Tick", msg :
      {
        // ensure that we are in careful mode and the user is not already being warned or already in
        // another radio conversation
        if ((carefulMode) and (!warningUser) and (!talkingToDriver))
        {
          warningUser = true;

          // The train is more than 10ph over speed limit, give warning and sound alarm
          Interface.SetObjectiveIcon("bad.tga");
          Sleep(World.Play2DSound("conversation6.wav"));
          Interface.AdjustScore(-50);
          Interface.Print(stringTable.GetString1("robe_bad_caution_speed", "50"));
          Interface.Log(stringTable.GetString1("robe_bad_caution_speed", "50"));

          // set up values so we can do a special check
          current_speed = loco.GetVelocity();
          current_speed = current_speed * MPS_TO_KPH;

          objectiveIcon = "10_mph.tga";
          Interface.SetObjectiveIcon("10_mph.tga");

          // sleep for a while, so warning messages to driver aren't reaplayed one after the other
          Sleep(10);
          warningUser = false;
        }
        continue;
      }
      on "Kill", "CautionZone", msg:
      {
        // kill message has been sent, get out of here
        Interface.Log(stringTable.GetString("robe_kill_caution"));
        break;
      }
    }

  }


  //
  // Warns user of their bad couple-up and displays a message along with alarm sound.
  //
  thread void BadCouple()
  {
    // set to true so this method won't be called repeatedly in near proximity
    recentBadCouple = true;

    // warn user and deduct points
    Interface.SetObjective(stringTable.GetString("std_bad_couple_imperial"), "bad.tga");
    Interface.AdjustScore(-50);
    World.Play2DSound("ALARM.wav");

    // recentDecouple reset after delay so we don't get 2 BadCouple() calls too close to each other
    Sleep(1.0);
    recentBadCouple = false;

    // after brief sleep, hide warning message and restore previous icon
    Sleep(3.0);
    Interface.SetObjective("", objectiveIcon);
    Interface.ShowObjective(false);
  }


  //
  // TrainBadCouple() is called by Trainz when vehicles couple greater than 8kph (5mph).
  //
  void TrainBadCouple(int vehicleId)
  {
    // was it a recent bad couple (last 3 seconds)? If so, display message and alarm
    if (!recentBadCouple) 
      BadCouple();
  }


  //
  // Manage first section of scenario where AI loaded train passes user at southern loop
  //
  void StartPassingLoop(void)
  {
    // start off the AI loaded ore train
    aiOreTrain.RunSchedule(aiLoadedTrainSchedule, false, 0);

    // display instruction sheet and setup thread to trap keystroke so it can be called back again
    DoInstructions();
    ManageBrowser();

    // user sits in LOOP siding waiting for loaded train to pass
    objectiveIcon = "pause.tga";
    Interface.SetObjectiveIcon("pause.tga");
    Timetable.OnAnnouncement(me, "LoadedClearJunction");
    Sleep(5);
    Navigate.LockJunction("LOOP South", Junction.DIRECTION_RIGHT, false);

    // icon gives user go-ahead to move forward
    objectiveIcon = "play.tga";
    Interface.SetObjectiveIcon("play.tga");

    // 200 points for passing as user pulls out of siding
    Navigate.OnTrigger(me, userOreTrain, "TRIGCONVERSATION1", Navigate.TRIGGER_ENTER);
    Interface.AdjustScore(200);
    Interface.Print(stringTable.GetString1("robe_objective_startloop", "200"));
    Interface.Log(stringTable.GetString1("robe_objective_startloop", "200"));

    // 500 points for arriving at Mesa
    Navigate.OnTrigger(me, userOreTrain, "TrigWarnMaint", Navigate.TRIGGER_ENTER);
    Interface.AdjustScore(500);
    Interface.Print(stringTable.GetString1("robe_objective_arrived_mesa", "500"));
    Interface.Log(stringTable.GetString1("robe_objective_arrived_mesa", "500"));
  }


  //
  // Handles the passing of the track crew at Mesa passing loop
  //
  void PassTrackCrew(Train trainToWatch, string entryTrigger, string exitTrigger)
  {
    //
    // User enters zone and informed by radio
    //

    // inform user by radio as they enter zone
    Navigate.OnTrigger(me, trainToWatch, entryTrigger, Navigate.TRIGGER_ENTER);
    talkingToDriver = true;

    // start special caution zone monitoring thread
    carefulMode = true;
    CautionZone(trainToWatch); 

    // display special 10mph temp speed limit icon
    objectiveIcon = "10_mph.tga";
    Interface.SetObjectiveIcon("10_mph.tga");

    // set a 3mph tolerance buffer on user train
    trainToWatch.SetFloatingLimit(3.8 * Train.KPH_TO_MPS, true);

    // play the warning radio message
    Sleep(World.Play2DSound("conversation5.wav"));
    talkingToDriver = false;


    // maintenance train blasts horn
    Navigate.OnTrigger(me, trainToWatch, "GangHorn", Navigate.TRIGGER_ENTER);
    Sleep(Math.Rand(1, 3));
    SendMessage(trackGangCrew, "loco", "horn");


    //
    // Train is clear, back to normal
    //
    Navigate.OnTrigger(me, trainToWatch, exitTrigger, Navigate.TRIGGER_LEAVE);
    talkingToDriver = true;

    // kill off special caution zone monitoring thread
    SendMessage(me, "Kill", "CautionZone"); 

    // update user with icon to go on
    objectiveIcon = "play.tga";
    Interface.SetObjectiveIcon("play.tga");

    // return tolerance level on speed limit back to normal
    trainToWatch.SetFloatingLimit(10.0 * Train.KPH_TO_MPS, true);

    // play radio message informaing user that they are clear
    Sleep(World.Play2DSound("conversation7.wav"));
    talkingToDriver = false;
    carefulMode = false;

    // 300 points for clearing the caution zone
    Interface.AdjustScore(300);
    Interface.Print(stringTable.GetString1("robe_objective_cautionzone", "300"));
    Interface.Log(stringTable.GetString1("robe_objective_cautionzone", "300"));
  }
  

  //
  // Manage run-around of the ore cars
  //
  void MesaRunAround(Train train)
  {
    bool runAroundDone = false;
    Vehicle[] tempArr = train.GetVehicles();
    Train oreWagons;
    Message msg;
    int trainsize = tempArr.size();


    //
    // Guide user into siding and uncouple locos from ore wagons
    //

    // warn user when in siding
    Navigate.OnTrigger(me, userOreTrain, "WarnMesaStop", Navigate.TRIGGER_ENTER);
    objectiveIcon = "pause.tga";
    Interface.SetObjectiveIcon("pause.tga");

    // warn user to stop
    Navigate.OnTrigger(me, userOreTrain, "MesaStop", Navigate.TRIGGER_ENTER);
    objectiveIcon = "stop.tga";
    Interface.SetObjectiveIcon("stop.tga");


    // wait for user to stop in MesaStop trigger (decouple zone)
    Navigate.OnTrigger(me, userOreTrain, "MesaStop", Navigate.TRIGGER_STOPPED); // checkpoint
    objectiveIcon = "decouple.tga";
    Interface.SetObjectiveIcon("decouple.tga");

    // do the decoupling from the ore train
    oreWagons = TrainUtil.DoDecouple(train, 40);
    userOreCars = oreWagons.GetVehicles();  // save ore wagons (later verification needed)


    //
    // Switch tracks and play conversation 2
    //

    // Set play icon after decouple
    objectiveIcon = "play.tga";
    Interface.SetObjectiveIcon("play.tga");

    // wait for user to exit junction and set stop icon
    Navigate.OnTrigger(me, train, "TrigWarnC2", Navigate.TRIGGER_LEAVE);
    objectiveIcon = "stop.tga";
    Interface.SetObjectiveIcon("stop.tga");

    // wait for user to reach the conversation 2 point (with only the two alcos?)
    Navigate.OnTrigger(me, train, "TRIGCONVERSATION2", Navigate.TRIGGER_STOPPED);
    objectiveIcon = "pause.tga";
    Interface.SetObjectiveIcon("pause.tga");

    // once train is stopped, we can do the second conversation
    train.SetAutopilotMode(Train.CONTROL_SCRIPT);
    train.SetDCCThrottle(0.0f);
    Sleep(Math.Rand(1, 4));
    Navigate.LockJunction("MESA J 4", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("MESA J 1", Junction.DIRECTION_RIGHT, false);

    // turn train around (both logically and switch user to the other cab)
    train.Turnaround();
    World.SetCamera(train.GetFrontmostLocomotive(), World.CAMERA_INTERNAL);
    Interface.Print(stringTable.GetString("robe_train_reversed"));
    Interface.Log(stringTable.GetString("robe_train_reversed"));

    // play cpnversation 2, once done, hand control back to user
    Sleep(World.Play2DSound("conversation2.wav"));
    train.SetAutopilotMode(Train.CONTROL_MANUAL);
    objectiveIcon = "play.tga";
    Interface.SetObjectiveIcon("play.tga");


    //
    // Run up siding and switch junctions to access ore cars
    //

    // manage junction switch at runaround
    Navigate.OnTrigger(me, train, "MesaJunc", Navigate.TRIGGER_ENTER);
    objectiveIcon = "stop.tga";
    Interface.SetObjectiveIcon("stop.tga");

    // change junction once stopped
    Navigate.OnTrigger(me, train, "MesaJunc", Navigate.TRIGGER_STOPPED);
    objectiveIcon = "back.tga";
    Interface.SetObjectiveIcon("back.tga");
    Navigate.LockJunction("MESA J 1", Junction.DIRECTION_LEFT, false);


    //
    // Couple loco up with ore cars (from rear)
    //

    // warn user to slow and couple up with ore cars
    Navigate.OnTrigger(me, train, "MesaJunc", Navigate.TRIGGER_LEAVE); // checkpoint
    objectiveIcon = "pause.tga";
    Interface.SetObjectiveIcon("pause.tga");

    Navigate.OnTrigger(me, train, "WarnMesaStop", Navigate.TRIGGER_ENTER);
    objectiveIcon = "decouple.tga";
    Interface.SetObjectiveIcon("decouple.tga");

    // couple up to ore cars from the other end
    TrainUtil.DoCoupleUp(train, oreWagons);

    // lock up coupling masks again on the complete ore train consist (needed as DoCoupleUp handles this?)
    train.SetCouplingMask(Vehicle.COUPLE_NONE);
    train.SetDecouplingMask(Vehicle.COUPLE_NONE);

    // done
    objectiveIcon = "play.tga";
    Interface.SetObjectiveIcon("play.tga");
  }


  //
  // Manages the lunch stop at Crew Quarters (after run-around)
  //
  void StopForLunch(void)
  {
    // warn user to stop near crew quarters
    Navigate.OnTrigger(me, userOreTrain, "TrigLunchStop", Navigate.TRIGGER_ENTER); 
    objectiveIcon = "stop.tga";
    Interface.SetObjectiveIcon("stop.tga");

    // wait for user to stop at lunch stop near crew quarters
    Navigate.OnTrigger(me, userOreTrain, "TrigLunchStop", Navigate.TRIGGER_STOPPED); 
    objectiveIcon = "lunch.tga";
    Interface.SetObjective(stringTable.GetString("robe_lunch_break"), "lunch.tga");

    // Play conversation 3 to start the lunch break
    Sleep(World.Play2DSound("conversation3.wav"));
    objectiveIcon = "lunch.tga";
    Interface.SetObjectiveIcon("lunch.tga");
    Interface.ShowObjective(false);
  }


  //
  // Handles the AI relief crew as they take train through mine while you have lunch
  //
  void ReliefCrew(void)
  {
    // Change to static tracking camera and drive loco consist away through to the mine
    World.SetCameraFlags(World.CAMERA_LOCKED);
    World.SetCamera(userOreTrain, World.CAMERA_TRACKING);
    Interface.Log("1");
    Sleep(Math.Rand(4,7));
    Interface.Log("2");

    // run the ore train manually under script control through to the mine
    userOreTrain.SetAutopilotMode(Train.CONTROL_SCRIPT);
    userOreTrain.SetHandBrake(false);
    userOreTrain.SetDCCThrottle(0.28);
    Navigate.OnTrigger(me, userOreTrain, "AISlowDown", Navigate.TRIGGER_ENTER);
    Interface.Log("3");
    userOreTrain.SetDCCThrottle(0.17);

    // trigger off the mine dumper particle effects
    Navigate.OnTrigger(me, userOreTrain, "TRIGFILLEFFECT", Navigate.TRIGGER_ENTER);
    Interface.Log("4");
    userOreTrain.SetDCCThrottle(0.0);

    // Activate particle effects. This is done by sending a message with the major of "pfx" and
    // minor of "+0 +1" to the mine gameobject which has particle emitters. The "+0 +1" minor 
    // string signals the two particle emitters to be active (i.e. The particle emitter number
    // prefixed by '+' switches that emitter on).
    SendMessage(mineParticle, "pfx", "+0 +1");
    Interface.Log("5");
    Interface.Log(stringTable.GetString("robe_mine_particles"));

    // sleep a while so user sees particles and than fade the screen to black
    Sleep(15); 
    BrowserInterface.SetOverlayColor(0.0, 0.0, 0.0, 1.0, 5.0);
    Sleep(5.0);


    // Now that we have faded to black, kill the particle effects and train. Deactivating the 
    // particle emitters is the same as activating them, except that the '-' prefix is used in the
    // minor string which switches off the emitter.
    SendMessage(mineParticle, "pfx", "-0 -1");
    World.DeleteTrain(userOreTrain);
  }


  //
  // Conversation #4 before heading back to Passing loop with heavy loaded tree
  //
  void AfterLunch(void)
  {
    // change time of day to approx 2pm
    World.SetGameTime(0.083);

    // create new loaded train (illusion of having been filled!)
    userOreTrain2 = World1.CreateTrain("RobeOreLoadedUser", "START HERE", true);
    userOreTrain.SetFloatingLimit(10.0 * Train.KPH_TO_MPS, true);  // set limit, so speeding penalties apply to the new train
    userOreTrain2.SetHandBrake(true);
    userOreTrain2.SetAutopilotMode(Train.CONTROL_SCRIPT);

    // switch to an internal view of new train consist before fading screen back in
    World.SetCameraFlags(World.CAMERA_NORMAL);
    World.SetCamera(userOreTrain2.GetFrontmostLocomotive(), World.CAMERA_INTERNAL);

    // fade screeen back to normal... (pause icon in place)
    objectiveIcon = "pause.tga";
    Interface.SetObjectiveIcon("pause.tga");
    Interface.ShowObjective(false);
    BrowserInterface.SetOverlayColor(0.0, 0.0, 0.0, 0.0, 5.0);
    Sleep(5.0);

    // play the 4th conversation 
    Sleep(Math.Rand(3, 5));
    Sleep(World.Play2DSound("conversation4.wav"));

    // give control back to user and head off....
    userOreTrain2.SetAutopilotMode(Train.CONTROL_MANUAL);
    userOreTrain2.SetFloatingLimit(10.0 * Train.KPH_TO_MPS, true);
    userOreTrain2.SetHandBrake(false); // release handbrake
    objectiveIcon = "play.tga";
    Interface.SetObjectiveIcon("play.tga");

    // 300 points for switching and lunch break
    Sleep(Math.Rand(2, 4));
    Interface.AdjustScore(300);
    Interface.Print(stringTable.GetString1("robe_objective_switch_lunch", "300"));
    Interface.Log(stringTable.GetString1("robe_objective_switch_lunch", "300"));
  }


  //
  // Journey back to south passing loop
  //
  void JourneyToLoop(void)
  {
    // create idle ore train waiting at passing loop
    aiOreTrain = World1.CreateTrain("robeoreemptyai", "StartUserEmpty", true);
    Navigate.LockJunction("LOOP South", Junction.DIRECTION_LEFT, false);

    // trigger off waiting AI empty ore train to blast its horn twice when passing around the 'LOOP South' junction
    Navigate.OnTrigger(me, userOreTrain2, "TRIGCONVERSATION1", Navigate.TRIGGER_ENTER);
    Sleep(Math.Rand(0, 2));
    SendMessage(aiOreTrain, "loco", "horn");

    Navigate.OnTrigger(me, userOreTrain2, "TrigLoopSouthClear", Navigate.TRIGGER_ENTER);
    Sleep(Math.Rand(2, 4));
    SendMessage(aiOreTrain, "loco", "horn");

    // 500 points for taking loaded ore train back the passing loop
    Sleep(Math.Rand(4, 6));
    Interface.AdjustScore(500);
    Interface.Print(stringTable.GetString1("robe_objective_endloop", "500"));
    Interface.Log(stringTable.GetString1("robe_objective_endloop", "500"));

    // trigger end of scenario
    Navigate.OnTrigger(me, userOreTrain2, "LoadedClear", Navigate.TRIGGER_ENTER);
  }


  //
  // main thread
  // brief main is executed automatically after Load() is called.  edit
  // main to contain your scenarios gameplay.
  //
  thread void main(void)
  {
    Train droppedOff;

    // Start the monitor thread to monitor speeding, derailing etc.
    Monitor();

    // Start off with 1000 points.  This will give us some points to play with before
    // the user obtains a negative score (if they do things wrong)
    Interface.AdjustScore(1000);

    // change time of day to approx 11am
    World.SetGameTime(0.978);

    // set camera to watch user train (internal view)
    World.SetCameraFlags(World.CAMERA_NORMAL);
    World.SetCamera(userOreTrain, World.CAMERA_INTERNAL);
    Sleep(3);

    // Let user select DCC or Cabin Control for the scenario
    SelectCabinDCC(userOreTrain);
    Sleep(4);


    //
    // Gameplay
    //

    // User idles at loop while loaded ai train passes with friendly sheep warning
    StartPassingLoop();

    // Slow passing by Track Maintenance Crew
    PassTrackCrew(userOreTrain, "TrigWarnCrew1", "TrigWarnCrew2");

    // Arival at Mesa
    MesaRunAround(userOreTrain);

    // Lunchbreak and Brew(tm) section
    StopForLunch();

    // Relief crew taking train through mine (while you are having lunch!)
    ReliefCrew();

    // Conversation 4 before heading back to Passing loop with heavy loaded tree
    AfterLunch();

    // Pass track maintanance crew again (on way back this time!)
    PassTrackCrew(userOreTrain2, "TrigWarnCrew2", "TrigWarnCrew1");

    // Journey back to south passing loop
    JourneyToLoop();


    // finish off the scenario
    Interface.SetObjective(stringTable.GetString1("scenario_complete_objective", Interface.GetScore()), "tick.tga");
    Interface.SetResults();
    Interface.LogResult(stringTable.GetString1("scenario_complete", Interface.GetScore()));
    Interface.Print(stringTable.GetString1("scenario_complete", Interface.GetScore()));
    scenarioDone = true;
    World.EndScenario(8.0);
  }
};
