//
// USASteelMillSwitching.gs
//
//  Copyright (C) 2002 Auran Developments Pty Ltd
//  All Rights Reserved.
//

include "trainz.gs"
include "world1.gs"
include "common.gs"
include "browser.gs"
include "stringtable.gs"


//
// Custom commands for AI trains
//

// Change junctions for empty coal train so it can get through the mill
class CoalTrainStart isclass CustomCommand
{
  bool Execute(Train train, int px, int py, int pz)
  {
    Navigate.LockJunction("FURNACE 03", Junction.DIRECTION_RIGHT, true);
    Navigate.LockJunction("MILLS TRACK", Junction.DIRECTION_RIGHT, true);
    Navigate.LockJunction("MINERALS YARD", Junction.DIRECTION_RIGHT, true);
    Navigate.LockJunction("HIGH LINE", Junction.DIRECTION_RIGHT, true);
    return true;
  }
};

// Sends message to train to blast it's horn
class AiHornMsg isclass CustomCommand
{
  bool Execute(Train train, int px, int py, int pz)
  {
    train.Sleep(Math.Rand(px, py));
    train.SendMessage(train, "loco", "horn");
    return true;
  }
};


//
// class MyUSAShunt
// brief This is the scenario class.
//
game class MyUSAShunt isclass Scenario, ScenarioSupport
{
  bool scenarioDone = false;
  bool recentBadCouple = false;
  string objectiveIcon = null;

  Train switcher, tipperConsist1, tipperConsist2, coalTrain, foundaryCars, pickupCars, alcos;
  Signal millSignal, loopSignal, exitSignal, entrySignal1, entrySignal2;
  Schedule coalTrainSchedule, alcoSchedule;
  CoalTrainStart coalTrainStart;
  AiHornMsg aiHorn;
  Vehicle[] droppedOff;
  Vehicle loco;               // pointer to user's shunting loco

  GameObject[] slagParticles; // array of scenery objects for particle emitters at slah heap
  StringTable stringTable;    // scenario string table


  //
  // Load will be called by Trainz to load the scenario map, and when the user presses Ctrl-L
  // param data is the save game data if loading a saved game.
  //
  bool Load(string data)
  {
    int i; // array index
    string tempStr;

    if(data and data.size())
    {
      Interface.Load(data);
    }

    // initialize scenario string table
    stringTable = ScenarioStringTable.Get();

    // load the map
    if(!World.LoadMap(World.FindKUID("NorthAmerica")))
    {
      Interface.Log(stringTable.GetString1("std_error_loading_map_name", "NorthAmerica"));
      return false;
    }

    // disable saving on this scenario
    BrowserInterface.SetSaveAllowed(false);

    // initialize gameobjecrts to access the slag heap particle emmitters
    slagParticles = new GameObject[8];
    for (i = 0; i < 8; i++)
    {
      tempStr = "Slag" + i;
      slagParticles[i] = Router.GetGameObject(tempStr);
      if (!slagParticles[i])
      {
        Interface.Log(stringTable.GetString1("std_error_loading_particleFX", "Slag Steam:" + tempStr));
        return false;
      }
    }


    //
    // Create train consists on the layout
    //

    // individual shunting loco
    switcher = World1.CreateTrain("SW7Switcher", "Start Furnace", true);
    switcher.SetFloatingLimit(10.0 * Train.KPH_TO_MPS, true);
    switcher.SetAutopilotMode(Train.CONTROL_SCRIPT); // disable user control until we are ready to roll later on
    loco = switcher.GetFrontmostLocomotive();

    // 2 consists of loaded slag tipper cars
    tipperConsist1 = World1.CreateTrain("LoadedSlagTippers", "Start Furnace1", true);
    tipperConsist2 = World1.CreateTrain("LoadedSlagTippers", "Start Furnace2", true);

    // cars for pickup in factory/shed
    pickupCars = World1.CreateTrain("freightpickup", "Start Pickup", true);


    //
    // Schedules
    //

    // Empty coal train schedule
    coalTrainSchedule = new Schedule();
    aiHorn = new AiHornMsg();
    coalTrainStart = new CoalTrainStart();

    coalTrainSchedule.Custom(coalTrainStart);
    coalTrainSchedule.OnTrigger("TrigLS", true);
    coalTrainSchedule.SetSignal("LS", Signal.RED, "Loop signal.");
    coalTrainSchedule.Custom(aiHorn, 0, 3, 0);
    coalTrainSchedule.OnTrigger("TrigDS", true);
    coalTrainSchedule.SetSignal("MS", Signal.RED, "Mill signal.");
    coalTrainSchedule.SetSignal("LS", Signal.AUTOMATIC, "Mill signal.");
    coalTrainSchedule.Custom(aiHorn, 3, 5, 0);
    coalTrainSchedule.OnTrigger("TrigClear", true);
    coalTrainSchedule.Custom(aiHorn, 2, 4, 0);
    coalTrainSchedule.SetSignal("ME", Signal.RED, "Mill signal.");
    coalTrainSchedule.SetSignal("MS", Signal.YELLOW, "Mill signal.");
    coalTrainSchedule.Announce("EmptyCoalTrainClear");
    coalTrainSchedule.OnTrigger("Trig KillCoalTrain", true);
    coalTrainSchedule.SetSignal("ME", Signal.AUTOMATIC, "Mill signal.");
    coalTrainSchedule.DeleteTrain();

    // schedule for Lehigh Valley alcos that haul up at end (for pickup of box cars)
    alcoSchedule = new Schedule();

    alcoSchedule.TakeJunction("HIGH LINE", Junction.DIRECTION_RIGHT);
    alcoSchedule.Custom(aiHorn, 1, 3, 0);
    alcoSchedule.TakeJunction("MINERALS YARD", Junction.DIRECTION_LEFT);
    alcoSchedule.StopAtTrigger("StopAlcos", 0);
    alcoSchedule.Custom(aiHorn, 2, 4, 0);


    //
    // Junctions
    //

    // entrance to mill and high line
    Navigate.LockJunction("Steel_Mills", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("HIGH LINE", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("MINERALS YARD", Junction.DIRECTION_LEFT, true);
    Navigate.LockJunction("Coke_Oven", Junction.DIRECTION_LEFT, true);

    // blast furnace switching area
    Navigate.LockJunction("Slag_Heap", Junction.DIRECTION_LEFT, true);
    Navigate.LockJunction("FURNACE 01", Junction.DIRECTION_RIGHT, true);
    Navigate.LockJunction("FURNACE 02", Junction.DIRECTION_RIGHT, true);
    Navigate.LockJunction("Quencher", Junction.DIRECTION_LEFT, true);

    // route for highline loop to pass through steel mill
    Navigate.LockJunction("FURNACE 03", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("MILLS TRACK", Junction.DIRECTION_LEFT, false);


    //
    // Signals
    //

    // get pointers to signal objects
    millSignal = cast<Signal> Router.GetGameObject("MS");
    loopSignal = cast<Signal> Router.GetGameObject("LS");
    exitSignal = cast<Signal> Router.GetGameObject("ME");
    entrySignal1 = cast<Signal> Router.GetGameObject("ES1");
    entrySignal2 = cast<Signal> Router.GetGameObject("ES2");

    // set to yellow so empty coal train passes through slowly
    millSignal.SetSignalState(Signal.YELLOW, "Caution!");
    loopSignal.SetSignalState(Signal.YELLOW, "Caution!");
    exitSignal.SetSignalState(Signal.YELLOW, "Caution!");


    return true;
  }


  //
  // Save will be called by Trainz when the user presses Ctrl-S.
  // return the save game string, such that load will be able to restor the save game
  // from the last save check point.
  //  
  string Save()
  {
    return Interface.Save();
  }


  //
  // TrainDerailed() will be called by Trainz when a train derails
  //
  void TrainDerailed(int trainId) 
  {
    if(!scenarioDone)
    {
      Interface.SetObjective(stringTable.GetString("std_terminated_derailed_train"), "bad.tga");
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);
      scenarioDone = true;
    }
  }


  //
  // TrainCollided() will be called by Trainz when a train collides
  //
  void TrainCollided(int trainId) 
  {
    if(!scenarioDone)
    {
      Interface.SetObjective(stringTable.GetString("std_terminated_train_collision"), "bad.tga");       
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);  
      scenarioDone = true;
    }
  }


  //
  // TrainSpeedingFine() is called by Trainz every second your trains speed exceeds the floating limit
  //
  void TrainSpeedingFine() 
  {
    Interface.AdjustScore(-1);  
  }


  //
  // Warns user of their bad couple-up and displays a message along with alarm sound.
  //
  thread void BadCouple()
  {
    // set to true so this method won't be called repeatedly in near proximity
    recentBadCouple = true;

    // warn user and deduct points
    Interface.SetObjective(stringTable.GetString("std_bad_couple_imperial"), "bad.tga");
    Interface.AdjustScore(-50);
    World.Play2DSound("ALARM.wav");

    // recentDecouple reset after delay so we don't get 2 BadCouple() calls too close to each other
    Sleep(1.0);
    recentBadCouple = false;

    // after brief sleep, hide warning message and restore previous icon
    Sleep(3.0);
    Interface.SetObjective("", objectiveIcon);
    Interface.ShowObjective(false);
  }


  //
  // TrainBadCouple() is called by Trainz when vehicles couple greater than 8kph (5mph).
  //
  void TrainBadCouple(int vehicleId)
  {
    // was it a recent bad couple? If so, display message and alarm.
    if (!recentBadCouple) 
      BadCouple();
  }


  //
  // Thread that handles the empty coal train that passes through
  //
  thread void RunEmptyCoalTrain(Schedule schedule)
  {
    // start the coal train after a random delay
    coalTrain.Sleep(Math.Rand(25, 60));
    coalTrain.RunSchedule(coalTrainSchedule, false, 0);

    // wait for empty coal train to clear mill so user can cross over with foundary cars
    Timetable.OnAnnouncement(me, "EmptyCoalTrainClear");
    Navigate.LockJunction("FURNACE 03", Junction.DIRECTION_LEFT, true);
    Navigate.LockJunction("MILLS TRACK", Junction.DIRECTION_LEFT, true);
    Navigate.LockJunction("Open_Hearth", Junction.DIRECTION_RIGHT, true);
    Interface.Print(stringTable.GetString("steel_empty_coal_clear"));
    Interface.Log(stringTable.GetString("steel_empty_coal_clear"));
  }


  //
  // Initialise the a tracking array to drop off the tippers at the slag heap.
  // Note: Assumes consists with tipper cars have already been created!
  //
  void InitSlagHeapTracking(ShuntTrack[] tracking)
  {
    int i; 
    Vehicle[] tempVehicles;

    // first half (3 cars)
    tempVehicles = tipperConsist1.GetVehicles();
    for (i = 0; i < tempVehicles.size(); i++)
    {
      tracking[i].vehicle = tempVehicles[i];
      tracking[i].trigger = cast<Trigger> Router.GetGameObject("Trig SlagHeap");
      tracking[i].name    = "Tipper " + (i+1);
    }

    // second half (3 cars)
    tempVehicles = tipperConsist2.GetVehicles();
    for (i = tempVehicles.size(); i < tracking.size(); i++)
    {
      tracking[i].vehicle = tempVehicles[i-3];
      tracking[i].trigger = cast<Trigger> Router.GetGameObject("Trig SlagHeap");
      tracking[i].name    = "Tipper " + (i+1);
    }
  }


  //
  // Initialize tracking array for empty slag tippers to be delivered at blast furnace.
  // Note: assumes that new consist is empty tippers has been created and pointed to by switcher
  //
  void InitBlastFurnaceTracking(ShuntTrack[] tracking)
  {
    int i;
    Vehicle[] tempVehicles;

    tempVehicles = switcher.GetVehicles();
    for (i = 0; i < tracking.size(); i++)
    {
      tracking[i].vehicle = tempVehicles[i+1];
      tracking[i].trigger = null;
      tracking[i].name = "Tipper " + (i+1);
    }
  }


  //
  // Manage stopping at slag heap
  //
  void DoSlagHeap(ShuntTrack[] shuntTrack)
  {
    bool pickupDone = false;
    bool shuntingDone = false;
    Message msg;
    string status;

    Interface.Log("Start of DoSlagHeap()"); // test/debug


    // wait until switcher has all tipppers in its consist
    TrainUtil.WaitUntilHas(loco, shuntTrack);
    Interface.Print(stringTable.GetString("steel_tippers_pickedup"));
    Interface.Log(stringTable.GetString("steel_tippers_pickedup"));

    // ensure consist pointer to switcher train is current
    switcher = loco.GetMyTrain();

    // block off access to coupling/decoupling (attempting to Decouple during fadeout cause Trainz to crash!)
    switcher.SetDecouplingMask(Vehicle.COUPLE_NONE);
    switcher.SetCouplingMask(Vehicle.COUPLE_NONE);

    // wait for user to take train to slag heap and warn them to stop
    Navigate.OnTrigger(me, switcher, "Trig SlagHeap", Navigate.TRIGGER_ENTER);
    objectiveIcon = "pause.tga";
    Interface.SetObjectiveIcon("pause.tga");

    // loop until all cars are at the slag heap drop-zone
    Sniff(switcher, "Train", "", true);
    while (!shuntingDone)
    {
      // wait for a train stopped message (only verify tipper cars are in place on stopped message, wasteful otherwise)
      on "Train", "StoppedMoving", msg :
      {
        // verify stopped train is switcher
        if (msg.src == switcher)
          // if so, loop-control flag is set to state of whether entire switcher is in the slag heap zone
          shuntingDone = TrainUtil.IsTrainInTrigger(me, switcher, shuntTrack[0].trigger);
      }
    }
    Sniff(switcher, "Train", "", false);

    Interface.Log("End of DoSlagHeap()"); // test/debug
  }


  //
  // Drop slag cars off at blast furnace
  //
  void DoBlastFurnace(ShuntTrack[] shuntTrack)
  {
    bool shuntingDone = false;
    int i, furnacecount1, furnacecount2;
    Message msg;
    string status;
    Trigger furnace1 = cast<Trigger> Router.GetGameObject("Trig Furnace1Drop");
    Trigger furnace2 = cast<Trigger> Router.GetGameObject("Trig Furnace2Drop");

    Interface.Log("Start of DoBlastFurnace()");


    // Re-initialise the tracking array for taking empty tipper cars back to blast furnace
    InitBlastFurnaceTracking(shuntTrack);

    // Take empty tippers back to blast furnace
    objectiveIcon = "play.tga";
    Interface.SetObjectiveIcon("play.tga");


    // loop until shunting is done
    while (!shuntingDone)
    {
      // wait for a decouple message (we only check for decoupled messages, wasteful otherwise)
      on "Vehicle", "Decoupled", msg :
      {
        // reset state flags
        shuntingDone = true;
        furnacecount1 = 0;
        furnacecount2 = 0;

        // test each vehicle for their location
        for (i = 0; i < shuntTrack.size(); i++)
        {
          // the rules of completion are basically that as longs there are 3 wagons in each blast
          // furnace siding, it doesn't matter
          if (TrainUtil.IsInTrigger(me, shuntTrack[i].vehicle, furnace1))
          {
            furnacecount1++;
            if (furnacecount1 > 3)
              shuntingDone = false;
          }
          else if (TrainUtil.IsInTrigger(me, shuntTrack[i].vehicle, furnace2))
          {
            furnacecount2++;
            if (furnacecount2 > 3)
              shuntingDone = false;
          }
          else
            shuntingDone = false;
        }
      }
    }

    // inform user of completion
    Interface.Print(stringTable.GetString("steel_tippers_dropped"));
    Interface.Log(stringTable.GetString("steel_tippers_dropped"));

    Interface.Log("End of DoBlastFurnace()"); // test/debug
  }


  //
  // Activates all particle emmitters at the slag heap
  //
  void ActivateSlagParticles()
  {
    int i;

    // Iterate through each game object in the array and activate its single particle emitter.
    // This is done by sending a message with the major of "pfx" and minor of "+0" to the scenery
    // objects, each which has a single partile emitter. The "+0" minor  string signals the
    // particle emitter to activate.
    for (i = 0; i < 8; i++)
      SendMessage(slagParticles[i], "pfx", "+0");
  }


  //
  // Deactivates all particle emmitters at the slag heap
  //
  void DeactivateSlagParticles()
  {
    int i;

    // Iterate through each gameobject in the array and deactivate its single particle emitter.
    // This is done by sending a message with the major of "pfx" and minor of "-0" to the scenery
    // objects.
    for (i = 0; i < 8; i++)
      SendMessage(slagParticles[i], "pfx", "-0");
  }


  //
  // Manage the first round-trip to the slag heap.
  //
  void DoSlagHeapReturn1()
  {
    int i;
    Vehicle[] slagTipperConsist;
    ShuntTrack[] tipperTracking;

    Interface.Log("Start of DoSlagHeapReturn1()"); // test/debug


    // Initialise tracking mechanism for wagons
    tipperTracking = new ShuntTrack[6];
    for (i = 0; i < 6; i++)
      tipperTracking[i] = new ShuntTrack();

    InitSlagHeapTracking(tipperTracking);


    //
    // Pickup the tippers from the mill and take to slag heap
    //
    DoSlagHeap(tipperTracking);
    objectiveIcon = "stop.tga";
    Interface.SetObjectiveIcon("stop.tga");


    //
    // Screen black-out and train swap
    //
    switcher.SetAutopilotMode(Train.CONTROL_SCRIPT);

    // Fade to black and kill off user's train so we can replace it with one that has empty tippers
    BrowserInterface.SetOverlayColor(0.0, 0.0, 0.0, 1.0, 3.0);
    Sleep(3.0);

    // kill of user's train (with loaded tippers)
    World.DeleteTrain(switcher);

    // re-create user's train (with empty tippers)
    switcher = World1.CreateTrain("EmptySlagTippers", "Start EmptySlag", true);
    switcher.SetFloatingLimit(10.0 * Train.KPH_TO_MPS, true);  // set limit, so speeding penalties apply to the new train
    World.SetCamera(switcher, World.CAMERA_EXTERNAL);          // give external view (user can see particle stuff)

    // update the switcher loco pointer
    loco = switcher.GetFrontmostLocomotive();

    // Create the foundary consist and empty coal train consists
    foundaryCars = World1.CreateTrain("FoundaryCars", "Start Foundary", true);
    coalTrain = World1.CreateTrain("EmptyCoalTrain", "StartEmptyCoal", true);

    // Activate the slag heap particle emitters
    ActivateSlagParticles();

    // Fade back in and return user to switcher consist at slag heap
    BrowserInterface.SetOverlayColor(0.0, 0.0, 0.0, 0.0, 3.0);
    Sleep(3.0);


    //
    // Back to the blast furnace...
    //

    // put user into cycle until they drop tippers off
    DoBlastFurnace(tipperTracking);

    // Deactivate the slag heap particle emitters (so they are off after a while)
    DeactivateSlagParticles();

    // set up droppedOff to track the 6 dropped off tippers
    droppedOff = new Vehicle[6];
    for (i = 0; i < 6; i++)
      droppedOff[i] = tipperTracking[i].vehicle;

    Interface.Log("End of DoSlagHeapReturn1()"); // test/debug
  }


  //
  // Manage the second round-trip to the slag heap.
  //
  void DoSlagHeapReturn2()
  {
    int i;
    Train[] tempTrainArr = new Train[6];
    Vehicle[] slagTipperConsist;
    ShuntTrack[] tipperTracking;

    Interface.Log("Start of DoSlagHeapReturn2()");


    // Initialise tracking mechanism for wagons
    tipperTracking = new ShuntTrack[6];
    for (i = 0; i < 6; i++)
      tipperTracking[i] = new ShuntTrack();

    InitSlagHeapTracking(tipperTracking);


    //
    // Pickup the tippers from the mill and take to slag heap
    //
    DoSlagHeap(tipperTracking);
    objectiveIcon = "pause.tga";
    Interface.SetObjectiveIcon("pause.tga");


    //
    // Screen black-out and train swap
    //
    switcher.SetAutopilotMode(Train.CONTROL_SCRIPT);

    // Fade to black and kill off user's train so we can replace it with one that has empty tippers
    BrowserInterface.SetOverlayColor(0.0, 0.0, 0.0, 1.0, 3.0);
    Sleep(3.0);

    // kill of user's train (with loaded tippers)
    World.DeleteTrain(switcher);

    // re-create user's train (with empty tippers)
    switcher = World1.CreateTrain("EmptySlagTippers", "Start EmptySlag", true);
    switcher.SetFloatingLimit(10.0 * Train.KPH_TO_MPS, true);  // set limit, so speeding penalties apply to the new train
    World.SetCamera(switcher, World.CAMERA_EXTERNAL);          // user external view so user can see particles

    // update the loco pointer
    loco = switcher.GetFrontmostLocomotive();

    // activate the slag heap particle emitters
    ActivateSlagParticles();

    // Fade back in and return user to switcher consist at slag heap
    BrowserInterface.SetOverlayColor(0.0, 0.0, 0.0, 0.0, 3.0);
    Sleep(3.0);


    //
    // Back to the blast furnace...
    //

    // put user into cycle until they drop tippers off
    DoBlastFurnace(tipperTracking);

    // deactivate the slag heap particle emitters
    DeactivateSlagParticles();

    Interface.Log("End of DoSlagHeapReturn2()"); // test/debug
  }


  //
  // Handle drop of foundary cars to the mill
  //
  void DoFoundaryMill()
  {
    int i, wagoncount;
    bool shuntingDone = false;
    Message msg;
    string status;
    Vehicle[] foundArray;
    ShuntTrack[] foundaryTracking;

    Interface.Log("Start of DoFoundaryMill()"); // test/debug


    // allow foundary cars to be shunted with
    foundaryCars.SetCouplingMask(Vehicle.COUPLE_BOTH);
    foundaryCars.SetDecouplingMask(Vehicle.COUPLE_BOTH);

    // Initialise tracking mechanism for foundary cars
    foundArray = foundaryCars.GetVehicles();
    wagoncount = foundArray.size();
    foundaryTracking = new ShuntTrack[wagoncount];
    for (i = 0; i < wagoncount; i++)
    {
      foundaryTracking[i] = new ShuntTrack();
      foundaryTracking[i].vehicle = foundArray[i];
      foundaryTracking[i].trigger = cast<Trigger> Router.GetGameObject("Trig MillDrop");
      foundaryTracking[i].name    = "Foundary Car + " + (i+1);
    }


    // Start running the empty coal train through...
    RunEmptyCoalTrain(coalTrainSchedule);

    // loop until shunting is done
    while (!shuntingDone)
    {
      // wait for a train decoupled message (only check verify cars are in place for each decouple message - wasteful otherwise)
      on "Vehicle", "Decoupled", msg :
      {
        // set flag to true, any vehicle out of place will make it false
        shuntingDone = true;

        // test each vehicle for their location
        for (i = 0; i < wagoncount; i++)
        {
          if (!TrainUtil.IsInTrigger(me, foundaryTracking[i].vehicle, foundaryTracking[i].trigger))
            shuntingDone = false;
        }
      }
    }

    Interface.Log("End of DoFoundaryMill()"); // test/debug
  }


  //
  // Manage the box car drop-off
  //
  void DoBoxCarDrop()
  {
    bool shuntingDone = false;
    Message msg;
    int i, wagoncount;
    Vehicle[] boxcarArray;
    ShuntTrack[] boxcarTracking;

    Interface.Log("Start of DoBoxCarDrop()"); // test/debug

    // set junction for convinience of user
    Navigate.LockJunction("Open_Hearth", Junction.DIRECTION_LEFT, true);


    // set to yellow so Alcos can come in and reach their destination near pickup zone
    entrySignal1.SetSignalState(Signal.YELLOW, "Caution!");
    entrySignal2.SetSignalState(Signal.YELLOW, "Caution!");

    // create double alco consist and run its schedule so it appears to arrive near drop-off zone
    alcos = World1.CreateTrain("alcos", "StartAlcos", true);
    alcos.RunSchedule(alcoSchedule, false, 0);


    // Initialise tracking mechanism for box cars
    boxcarArray = pickupCars.GetVehicles();
    wagoncount = boxcarArray.size();
    boxcarTracking = new ShuntTrack[wagoncount];
    for (i = 0; i < wagoncount; i++)
    {
      boxcarTracking[i] = new ShuntTrack();
      boxcarTracking[i].vehicle = boxcarArray[0];
      boxcarTracking[i].trigger = cast<Trigger> Router.GetGameObject("TrigDropZone");
      boxcarTracking[i].name    = "Box Car " + (i+1);
    }

    // enable box car consist to be coupled to/decoupled from for switching
    pickupCars.SetCouplingMask(Vehicle.COUPLE_BOTH);
    pickupCars.SetDecouplingMask(Vehicle.COUPLE_BOTH);


    // loop until shunting is done
    while (!shuntingDone)
    {
      // wait for a train stopped message (only check for stopped messages)
      on "Vehicle", "Decoupled", msg :
      {
        // reset various states
        shuntingDone = true;  // set flag to true, any vehicle out of place will make it false

        // test each vehicle for their location
        for (i = 0; i < wagoncount; i++)
        {
          if (!TrainUtil.IsInTrigger(me, boxcarTracking[i].vehicle, boxcarTracking[i].trigger))
            shuntingDone = false;
        }
      }
    }

    Interface.Log("End of DoBoxCarDrop()"); // test/debug
  }
 

  //
  // main thread
  // brief main is executed automatically after Load() is called.
  //
  thread void main(void)
  {
    int i; // loop counter

    // Start the monitor thread to monitor speeding, derailing etc.
    Monitor();

    // Start off with 1000 points.  This will give us some points to play with before
    // the user obtains a negative score (if they do things wrong)
    Interface.AdjustScore(1000);


    //
    // Gameplay
    //

    // start with pause icon
    objectiveIcon = "pause.tga";
    Interface.SetObjectiveIcon("pause.tga");

    // Let user select DCC or Cabin Control for the scenario
    SelectCabinDCC(switcher);

    // display instruction sheet and setup thread to trap keystroke so it can be called back again
    DoInstructions();
    ManageBrowser();

    // intro radio message (train can't be controlled until radio message has been done - avoids 
    // problems with impatient user picking up slag cars too early)
    Sleep(Math.Rand(3, 5));
    Sleep(World.Play2DSound("slag_ready.wav"));
    objectiveIcon = "play.tga";
    Interface.SetObjectiveIcon("play.tga");
    switcher.SetAutopilotMode(Train.CONTROL_MANUAL);


    //
    // Return trip to slag heap #1
    //
    DoSlagHeapReturn1();

    // 300 points for the first slag heap trip
    Interface.AdjustScore(300);
    Interface.Print(stringTable.GetString2("steel_objective_slagheap", "1st", "300"));
    Interface.Log(stringTable.GetString2("steel_objective_slagheap", "1st", "300"));


    //
    // Pickup the foundary cars and take them to the mill
    //
    Sleep(Math.Rand(6, 10));
    Sleep(World.Play2DSound("foundry_ready.wav"));
    DoFoundaryMill();
    objectiveIcon = "play.tga";
    Interface.SetObjectiveIcon("play.tga");

    // 200 points for the foundary trip
    Interface.AdjustScore(200);
    Interface.Print(stringTable.GetString1("steel_objective_foundary", "200"));
    Interface.Log(stringTable.GetString1("steel_objective_foundary", "200"));


    //
    // Return trip to slag heap #2
    //

    // delete the previously dropped tippers before we do the 2nd slag heap trip
    for (i = 0; i < droppedOff.size(); i++)
      World.DeleteTrain(droppedOff[i].GetMyTrain());


    // place new loaded slag tippers in blast furnace
    tipperConsist1 = World1.CreateTrain("LoadedSlagTippers", "Start Furnace1", true);
    tipperConsist2 = World1.CreateTrain("LoadedSlagTippers", "Start Furnace2", true);

    Sleep(Math.Rand(6, 10));
    Sleep(World.Play2DSound("slag_ready2.wav"));
    DoSlagHeapReturn2();

    // 300 points for the second slag heap trip
    Interface.AdjustScore(300);
    Interface.Print(stringTable.GetString2("steel_objective_slagheap", "2nd", "300"));
    Interface.Log(stringTable.GetString2("steel_objective_slagheap", "2nd", "300"));


    //
    // Pickup the box cars and take them to drop-off zone
    //
    Sleep(Math.Rand(6, 10));
    Sleep(World.Play2DSound("drop_ready.wav"));
    DoBoxCarDrop();

    Sleep(Math.Rand(3, 8));
    Sleep(World.Play2DSound("zone_ready.wav"));

    // 200 points for the box car drop
    Interface.AdjustScore(200);
    Interface.Print(stringTable.GetString1("steel_objective_boxcar_drop", "200"));
    Interface.Log(stringTable.GetString1("steel_objective_boxcar_drop", "200"));
    Sleep(Math.Rand(3, 5));


    // finish off the scenario
    Interface.SetObjective(stringTable.GetString1("scenario_complete_objective", Interface.GetScore()), "tick.tga");
    Interface.SetResults();
    Interface.LogResult(stringTable.GetString1("scenario_complete", Interface.GetScore()));
    Interface.Print(stringTable.GetString1("scenario_complete", Interface.GetScore()));
    scenarioDone = true;
    World.EndScenario(8.0);
  }
};

